#ifndef E_UTIL_H
#include "eutil.h"
#endif




/*
--------------------------------------------------------------------------------
                                  CONSTRUCTOR
--------------------------------------------------------------------------------
*/
extern EButton::EButton( int type, int id )
{
    /*  The button's current position */
    ePos.x = 0;
    ePos.y = 0;

    /*  The index of the texture we want */
    eTextureIndex = 0;

    /*  Dimensions */
    eWidth = 0;
    eHeight = 0;

    /*  Whether or not the mouse is in or out */
    eActivated = false;

    /*  What type of button this is */
    eType = type;

    /*  The button's ID */
    eID = id;
}



/*
--------------------------------------------------------------------------------
                                   ACTION_RMB

                       Action for the right mouse button
--------------------------------------------------------------------------------
*/
extern void EButton::action_rmb( void )
{
    switch( eType )
    {
        case BT_PRESET:     //  A preset button
            init_time_screen( eID );
            changingTime = eID;
            currentScreen = SCREEN_TIME;

            /*  We do this to get rid if a 'highlight bug' */
            if( eID < TOTAL_PRESET_BUTTONS )
                gPresetButtons[ eID ]->set_highlight( false );
            else
                gAlarmButton->set_highlight( false );

            break;

        case BT_UP_HOUR:    //  UP button (hour)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_hour( 3 );
            }
            break;

        case BT_UP_MINUTE:  //  UP button (minute)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_min( 5 );
            }
            break;

        case BT_UP_SECOND:  //  UP button (second)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_sec( 5 );
            }
            break;

        case BT_DOWN_HOUR:    //  DOWN button (hour)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_hour( -3 );
            }
            break;

        case BT_DOWN_MINUTE:  //  DOWN button (minute)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_min( -5 );
            }
            break;

        case BT_DOWN_SECOND:  //  DOWN button (second)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_sec( -5 );
            }
            break;
    }
}


/*
--------------------------------------------------------------------------------
                                   ACTION_LMB
                        Action for the left mouse button
--------------------------------------------------------------------------------
*/
extern void EButton::action_lmb( void )
{
    /*  Alter tick according to button held */
    int tick = 1;

    /*  Holding CTRL */
    if( SDL_GetModState() & KMOD_CTRL )
    {
        if( eType == BT_UP_HOUR || eType == BT_DOWN_HOUR )
            tick = 3;
        else
            tick = 5;
    }

    /*  Holding SHIFT */
    else if( SDL_GetModState() & KMOD_SHIFT )
    {
        if( eType == BT_UP_HOUR || eType == BT_DOWN_HOUR )
            tick = 6;
        else
            tick = 15;
    }

    switch( eType )
    {
        case BT_PRESET:     //  PRESET button
            gTimes[ TIME_ALARM]->set_time(
                    gTimes[ eID ]->get_time() );
            init_alarm_time();
            break;

        case BT_MAIN:       //  MAIN button
            if( currentScreen != SCREEN_MAIN )
                currentScreen = SCREEN_MAIN;
            break;

        case BT_HELP:       //  HELP button
            if( currentScreen != SCREEN_HELP )
                currentScreen = SCREEN_HELP;
            break;

        case BT_ABOUT:      //  ABOUT button
            if( currentScreen != SCREEN_ABOUT )
                currentScreen = SCREEN_ABOUT;
            break;

        case BT_FORMAT:     //  FORMAT button
            /*  Toggle format */
            if( format12 == true )
                format12 = false;
            else
                format12 = true;

            init_time_buttons();
            layout_main_screen();
            break;

        case BT_GO:         //  GO button
            if( currentScreen != SCREEN_ALARM )
            {
                currentScreen = SCREEN_ALARM;
                init_alarm_time();
                update_system_time();
                layout_alarm_screen();
            }
            break;

        case BT_NO:         //  CANCEL button
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->restore_time();
                currentScreen = SCREEN_MAIN;
            }
            break;

        case BT_YES:        //  OKAY button
            {
                if( currentScreen == SCREEN_TIME )
                {
                    gTimes[ changingTime ]->set_time_tr();
                    init_time_buttons();
                    currentScreen = SCREEN_MAIN;
                }
            }
            break;

        case BT_UP_HOUR:    //  UP button (hour)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_hour( tick );
            }
            break;

        case BT_UP_MINUTE:  //  UP button (minute)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_min( tick );
            }
            break;

        case BT_UP_SECOND:  //  UP button (second)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_sec( tick );
            }
            break;

        case BT_DOWN_HOUR:    //  DOWN button (hour)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_hour( -tick );
            }
            break;

        case BT_DOWN_MINUTE:  //  DOWN button (minute)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_min( -tick );
            }
            break;

        case BT_DOWN_SECOND:  //  DOWN button (second)
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->tick_sec( -tick );
            }
            break;
    }
}



/*
--------------------------------------------------------------------------------
                                      INIT
--------------------------------------------------------------------------------
*/
extern void EButton::init( int x, int y, int w, int h, int index )
{
    /*  The button's current position */
    ePos.x = x;
    ePos.y = y;

    /*  The index of the texture we want */
    eTextureIndex = index;

    /*  Dimensions */
    eWidth = w;
    eHeight = h;
}



/*
--------------------------------------------------------------------------------
                                  SET_POSITION
--------------------------------------------------------------------------------
*/
extern void EButton::set_position( int x, int y )
{
    ePos.x = x;
    ePos.y = y;
}



/*
--------------------------------------------------------------------------------
                                 INSIDE_BUTTON
                  Determine if the mouse is inside the button
--------------------------------------------------------------------------------
*/
extern bool EButton::inside_button( void )
{
    /*  Find the mouse position */
    int x, y;
    SDL_GetMouseState( &x, &y );

    /*  Check if the mouse is within bounds of the current button */

    if( x < ePos.x )            //  Left of button
        return( false );
    if( x > ePos.x + eWidth )   //  Right of button
        return( false );
    if( y < ePos.y )            //  Above button
        return( false );
    if( y > ePos.y + eHeight )  //  Below button
        return( false );

    /*  We're good if we made it this far */
    return( true );

}



/*
--------------------------------------------------------------------------------
                                  HANDLE_EVENT
--------------------------------------------------------------------------------
*/
extern void EButton::handle_event( SDL_Event &e )
{
    /*  If we have a mousedown / mouseup event */
    if( e.type == SDL_MOUSEBUTTONDOWN )
    {
        /*  If the mouse is inside the button and it isn't activated */
        if( inside_button() )
        {
            /*  If the button isn't activated, activate it */
            if( ! eActivated )
                eActivated = true;
        }

        /*  If it's outside the button */
        else
        {
            eActivated = false;
        }
    }

    /*  If we have a mouseup event */
    else if( e.type == SDL_MOUSEBUTTONUP )
    {
        if( inside_button() )
        {
            /*  If the button is activated, deactivate it */
            if( eActivated )
                eActivated = false;
            
            /*  Perform action */
            if( e.button.button == SDL_BUTTON_LEFT )
                action_lmb();
            else if( e.button.button == SDL_BUTTON_RIGHT )
                action_rmb();
        }
        else
            eActivated = false;
    }

    /*  Key down event */
    else if( e.type == SDL_KEYDOWN )
    {
        /*  Alter the amount we tick based on which modifiers are held */
        int tick = 1;
        if( SDL_GetModState() & KMOD_CTRL )
        {
            if( changingTimeFocus == FOCUS_HOUR )
                tick *= 3;
            else
                tick *= 5;
        }
        else if( SDL_GetModState() & KMOD_SHIFT )
        {
            if( changingTimeFocus == FOCUS_HOUR )
                tick *= 6;
            else
                tick *= 15;
        }


        /*
         *  Up/down arrows increase or decrease the selected time slot by an
         *  amount equal to the 'tick' variable.
         */
        switch( e.key.keysym.sym )
        {
            case SDLK_UP:       //  UP  arrow

                /*  Currently focused on the hour slot */
                if( changingTimeFocus == FOCUS_HOUR )
                {
                    gTimeButtons[ T_BUTTON_UP1 ]->activate();
                    if( heldTicks == 0 )
                    {
                        gTimes[ changingTime ]->tick_hour( tick );
                        ++heldTicks;
                    }
                }

                /*  Currently focused on the minute slot */
                else if( changingTimeFocus == FOCUS_MINUTE )
                {
                    gTimeButtons[ T_BUTTON_UP2 ]->activate();
                    if( heldTicks == 0 )
                    {
                        gTimes[ changingTime ]->tick_min( tick );
                        ++heldTicks;
                    }
                }

                /*  Currently focued on the second slot */
                else if( changingTimeFocus == FOCUS_SECOND )
                {
                    gTimeButtons[ T_BUTTON_UP3 ]->activate();
                    if( heldTicks == 0 )
                    {
                        gTimes[ changingTime ]->tick_sec( tick );
                        ++heldTicks;
                    }
                }

                break;

            case SDLK_DOWN:     //  DOWN arrow
                if( changingTimeFocus == FOCUS_HOUR )
                {
                    gTimeButtons[ T_BUTTON_DOWN1 ]->activate();
                    if( heldTicks == 0 )
                    {
                        gTimes[ changingTime ]->tick_hour( -tick );
                        ++heldTicks;
                    }
                }

                else if( changingTimeFocus == FOCUS_MINUTE )
                {
                    gTimeButtons[ T_BUTTON_DOWN2 ]->activate();
                    if( heldTicks == 0 )
                    {
                        gTimes[ changingTime ]->tick_min( -tick );
                        ++heldTicks;
                    }
                }

                else if( changingTimeFocus == FOCUS_SECOND )
                {
                    gTimeButtons[ T_BUTTON_DOWN3 ]->activate();
                    if( heldTicks == 0 )
                    {
                        gTimes[ changingTime ]->tick_sec( -tick );
                        ++heldTicks;
                    }
                }

                break;

            /*
             *  Move left or right with the left/right mouse keys.  If we're
             *  all the way to the left/right, go 'around' to the other side
             */
            case SDLK_LEFT:     //  LEFT arrow
                if( changingTimeFocus == FOCUS_HOUR )
                    changingTimeFocus = FOCUS_SECOND;
                else
                    --changingTimeFocus;

                break;

            case SDLK_RIGHT:    //  RIGHT arrow
                if( changingTimeFocus == FOCUS_SECOND )
                    changingTimeFocus = FOCUS_HOUR;
                else
                    ++changingTimeFocus;

                break;

            case SDLK_ESCAPE:   //  ESCAPE key      ( cancel )
                gTimeButtons[ T_BUTTON_NO1 ]->activate();
                break;

            case SDLK_RETURN:   //  RETURN key      ( confirm )
                gTimeButtons[ T_BUTTON_YES1 ]->activate();
        }

    }

    /*  Key up event */
    else if( e.type == SDL_KEYUP )
    {
        /*  If this button is any of the time-altering arrow buttons */
        if( eType >= BT_UP_HOUR  &&  eType <= BT_DOWN_SECOND )
        {
            heldTicks = 0;
        }

        /*  Deactivate all time buttons on any key up event */
        for( int i = 0; i < TOTAL_TIME_BUTTONS; ++i )
        {
            gTimeButtons[ i ]->deactivate();
        }

        /*  Return to main menu */
        if( e.key.keysym.sym == SDLK_ESCAPE )   //  ESCAPE key
            currentScreen = SCREEN_MAIN;

        /*  Confirm edit */
        else if( e.key.keysym.sym == SDLK_RETURN )  //  RETURN key
        {
            if( currentScreen == SCREEN_TIME )
            {
                gTimes[ changingTime ]->set_time_tr();
                init_time_buttons();
                currentScreen = SCREEN_MAIN;
            }
        }
    }

}



/*
--------------------------------------------------------------------------------
                               SET_TEXTURE_INDEX
--------------------------------------------------------------------------------
*/
extern void EButton::set_texture_index( int index )
{
    eTextureIndex = index;
}



/*
--------------------------------------------------------------------------------
                                     RENDER
--------------------------------------------------------------------------------
*/
extern void EButton::render( void )
{
    /*  Show the current texture */
    if( ! eActivated )
        gButtonTextures[ eTextureIndex ]->render( ePos.x, ePos.y );
    else
        gButtonTextures[ eTextureIndex + 1 ]->render( ePos.x, ePos.y );
}



/*
--------------------------------------------------------------------------------
                                 HANDLE_TOOLTIP
--------------------------------------------------------------------------------
*/
extern void EButton::handle_tooltip( void )
{
    if( inside_button() )
    {
        switch( eType )
        {
            case BT_PRESET:     //  PRESET BUTTON
                if( eID < TOTAL_PRESET_BUTTONS )
                    currentToolTip = TOOLTIP_PRESET;
                else
                    currentToolTip = TOOLTIP_ALARM;
                break;

            case BT_MAIN:       //  MAIN BUTTON
                currentToolTip = TOOLTIP_MAIN;
                break;

            case BT_HELP:       //  HELP BUTTON
                currentToolTip = TOOLTIP_HELP;
                break;

            case BT_ABOUT:       //  ABOUT BUTTON
                currentToolTip = TOOLTIP_ABOUT;
                break;

            case BT_FORMAT:       //  FORMAT BUTTON
                currentToolTip = TOOLTIP_FORMAT;
                break;

            case BT_GO:       //  GO BUTTON
                currentToolTip = TOOLTIP_GO;
                break;

        }
    }
}



/*
--------------------------------------------------------------------------------

                               ACCESSOR FUNCTIONS

--------------------------------------------------------------------------------
*/


/*  get width */
extern int EButton::get_width( void )
{
    return( eWidth );
}


/*  Set width */
extern void EButton::set_width( int width )
{
    eWidth = width;
}


/*  Get height */
extern int EButton::get_height( void )
{
    return( eHeight );
}


/*  Set height */
extern void EButton::set_height( int height )
{
    eHeight = height;
}


/*  Get button type */
extern int EButton::get_type( void )
{
    return( eType );
}


/*  Get X position */
extern int EButton::get_pos_x( void )
{
    return( ePos.x );
}


/*  Get Y position */
extern int EButton::get_pos_y( void )
{
    return( ePos.y );
}


/*  Get button ID */
extern int EButton::get_id( void )
{
    return( eID );
}


/*  Set button type */
extern void EButton::set_type( int type )
{
    eType = type;
}


/*  Set button ID */
extern void EButton::set_id( int id )
{
    eID = id;
}


/*  Set the button to activated status */
extern void EButton::activate( void )
{
    eActivated = true;
}


/*  Set the button to deactivated status */
extern void EButton::deactivate( void )
{
    eActivated = false;
}
