/*------------------------------------------------------------------------------
 *  The button class is used for on-screen buttons (surprise).  By default, it
 *  has two graphics, one for its deactivated state (stored in eTextureIndex)
 *  and another for its activated state.  The actions that occur when the button
 *  is pressed depend on which type of button it is, its ID, and which event
 *  occurred.
 *----------------------------------------------------------------------------*/
#ifndef E_CLASS_BUTTON_H
#define E_CLASS_BUTTON_H

/*
 *  Button class
 */
class EButton
{
    public:
        /*  Constructor */
        EButton( int type = -1, int id = -1 );

        /*  Initialization function */
        void init( int x, int y, int w, int h, int index = 0 );

        /*  Handle events */
        void handle_event( SDL_Event &e );

        /*  Check if the mouse is inside the button */
        bool inside_button( void );

        /*  Render meta-function */
        void render( void );

        /*  Set the position */
        void set_position( int x, int y );

        /*  Get the position */
        int get_pos_x( void );
        int get_pos_y( void );

        /*  Set the texture index */
        void set_texture_index( int index );

        /*  Set or return button dimensions */
        void set_width( int width );
        void set_height( int height );
        int get_width( void );
        int get_height( void );

        /*  Set whether or not the texture is graphical */
        void set_graphical( bool isGraphical );

        /*  The action this button performs */
        void action_lmb( void );

        /*  The alternate action for a button */
        void action_rmb( void );

        /*  Get or set the button type */
        int get_type( void );
        void set_type( int type );

        /*  Get or set the button ID */
        int get_id( void );
        void set_id( int id );

        /*  Handle the tooltips */
        void handle_tooltip( void );

        /*  Activate / deactivate the button */
        void activate( void );
        void deactivate( void );


    protected:
        /*  The position */
        SDL_Point ePos;

        /*  The index of the texture we want */
        int eTextureIndex;

        /*  The button's dimensions */
        int eWidth;
        int eHeight;

        /*  Whether or not the button has been activated */
        bool eActivated;

        /*  What type of button this is */
        int eType;

        /*  This button's ID */
        int eID;
};

#endif
