/*------------------------------------------------------------------------------
 *  This file is where we set the position for all of the various buttons,
 *  textures and rectangles that are used elsewhere.
 *----------------------------------------------------------------------------*/
#ifndef E_UTIL_H
#include "eutil.h"
#endif



/*
--------------------------------------------------------------------------------
                                  GET_INIT_POS

                         Function to center the buttons
--------------------------------------------------------------------------------
*/
int get_init_pos( int maxHeight, int numButtons, int height, int spacing )
{
    /*  Get the initial position */
    int pos = maxHeight - ( numButtons * height ) - ( numButtons * spacing );

    /*  Correct for individual button spacing */
    pos -= spacing/2;

    return( pos );
}



/*
--------------------------------------------------------------------------------
                             LAYOUT_PRESET_BUTTONS
--------------------------------------------------------------------------------
*/
extern void layout_preset_buttons( void )
{
    /*  Default position / dimensions for preset buttons */
    int x = 10;
    int y;
    int w = 160;
    int h = 40;

    /*  For all of our preset buttons */
    for( int i = 0; i < TOTAL_PRESET_BUTTONS; ++i )
    {
        y = ( i * h ) + ( i * (h/4) ) + 20;

        /*  Init the x/y pos, w/h dimensions and the texture index */
        gPresetButtons[ i ]->init( x, y, w, h, i );

        /*  Set the preset rect */
        presetRect.x = x;
        presetRect.y = y;
        presetRect.w = w;
        presetRect.h = h;

        /*  Init the button's draw rect */
        gPresetButtons[ i ]->init_draw_rect( &presetRect );

        /*  Set the text position */
        if ( ! format12 )  //  24-Hour format
            gTextTextures[ i ]->set_position( x + 18, y + 2 );
        else                //  12-Hour format
            gTextTextures[ i ]->set_position( x + 12, y + 5 );
    }
}



/*
--------------------------------------------------------------------------------
                            LAYOUT_GRAPHICAL_BUTTONS
--------------------------------------------------------------------------------
*/
extern void layout_graphical_buttons( void )
{
    /*  Position of the GO button */
    gMainButtons[ T_BUTTON_GO ]->set_position( 
            (SCREEN_WIDTH) - gMainButtons[ T_BUTTON_GO ]->get_width(),
            (SCREEN_HEIGHT) - gMainButtons[ T_BUTTON_GO ]->get_height() );

    /*  Position of the MAIN button */
    gMainButtons[ T_BUTTON_MAIN ]->set_position(
            (SCREEN_WIDTH) - gMainButtons[ T_BUTTON_MAIN ]->get_width(), 20 );

    /*  Position of the HELP button */
    gMainButtons[ T_BUTTON_HELP ]->set_position(
            (SCREEN_WIDTH) - gMainButtons[ T_BUTTON_HELP ]->get_width(), 100 );

    /*  Position of the ABOUT button */
    gMainButtons[ T_BUTTON_ABOUT ]->set_position(
            (SCREEN_WIDTH) - gMainButtons[ T_BUTTON_ABOUT ]->get_width(), 180 );

    /*  Position of the FORMAT button */
    gMainButtons[ T_BUTTON_FORMAT ]->set_position(
            (SCREEN_WIDTH) - gMainButtons[ T_BUTTON_FORMAT ]->get_width(), 260);

}



/*
--------------------------------------------------------------------------------
                               LAYOUT_TIME_SCREEN
--------------------------------------------------------------------------------
*/
extern void layout_time_screen( void )
{
    /*  Position of the time screen title */
    gTextTextures[ T_TEXT_TIME_TITLE ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ T_TEXT_TIME_TITLE]->get_width())
              / 2, 0 );

    /*  Storing the index here to save typing.  Yes, I am lazy. */
    int i;

    if( ! format12 )       //  24-Hour format
    {
        /*  Up arrows */
        i = T_BUTTON_UP1;
        gTimeButtons[ i ]->set_position( 270, 150 );
        gTimeButtons[ i + 1 ]->set_position( 375, 150 );
        gTimeButtons[ i + 2 ]->set_position( 480, 150 );

        /*  Down arrows */
        i = T_BUTTON_DOWN1;
        gTimeButtons[ i ]->set_position( 270, 268 );
        gTimeButtons[ i + 1 ]->set_position( 375, 268 );
        gTimeButtons[ i + 2 ]->set_position( 480, 268 );
    }

    else                    //  12-Hour format
    {
        i = T_BUTTON_UP1;
        gTimeButtons[ i ]->set_position( 250, 150 );
        gTimeButtons[ i + 1 ]->set_position( 335, 150 );
        gTimeButtons[ i + 2 ]->set_position( 420, 150 );

        /*  Down arrows */
        i = T_BUTTON_DOWN1;
        gTimeButtons[ i ]->set_position( 250, 268 );
        gTimeButtons[ i + 1 ]->set_position( 335, 268 );
        gTimeButtons[ i + 2 ]->set_position( 420, 268 );
    }
    

    /*  The okay button */
    gTimeButtons[ T_BUTTON_YES1 ]->set_position(
            ( (SCREEN_WIDTH) - gTimeButtons[ T_BUTTON_YES1 ]->get_width() ),
            ( (SCREEN_HEIGHT) - gTimeButtons[ T_BUTTON_YES1 ]->get_height() ) );

    /*  The cancel button */
    gTimeButtons[ T_BUTTON_NO1 ]->set_position( 0,
            ( (SCREEN_HEIGHT) - gTimeButtons[ T_BUTTON_NO1 ]->get_height() ) );

    /*  Center the time screen time texture */
    gTextTextures[ T_TEXT_TIME_TIME ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ T_TEXT_TIME_TIME ]->get_width())
            /2, ( ( (SCREEN_HEIGHT) - gTextTextures[ T_TEXT_TIME_TIME]->
                get_height() ) /2) - 30 );


    /*  Adjust the position of the rect for the time slots */
    i = changingTimeFocus;

    timeSlotRect.x = gTimeButtons[ i ]->get_pos_x() - 10;
    timeSlotRect.y = gTimeButtons[ i ]->get_pos_y() + 
        gTimeButtons[ i ]->get_height();
    timeSlotRect.w = 70;
    timeSlotRect.h = gTimeButtons[ i + 3 ]->get_pos_y() - timeSlotRect.y;


    /*  Adjust the positions of the time screen hints */
    i = T_TEXT_TIME_HINT1;
    gTextTextures[ i ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ i ]->get_width() ) / 2,
            gTimeButtons[ T_BUTTON_UP1 ]->get_pos_y() - 50 );

    i = T_TEXT_TIME_HINT2;
    gTextTextures[ i ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ i ]->get_width() ) / 2,
            gTimeButtons[ T_BUTTON_DOWN1 ]->get_pos_y() +
            gTimeButtons[ T_BUTTON_DOWN1 ]->get_height() + 40 );

    i = T_TEXT_TIME_HINT3;
    gTextTextures[ i ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ i ]->get_width() ) / 2,
            gTimeButtons[ T_BUTTON_DOWN1 ]->get_pos_y() +
            gTimeButtons[ T_BUTTON_DOWN1 ]->get_height() + 70 );


}



/*
--------------------------------------------------------------------------------
                               LAYOUT_MAIN_SCREEN
--------------------------------------------------------------------------------
*/
extern void layout_main_screen( void )
{
    /*  I'm going to use this to save my fingers */
    int index = 0;

    /*  Position all of the graphical buttons */
    layout_graphical_buttons();

    /*  Position the preset buttons */
    layout_preset_buttons();

    /*  Position the system time */
    index = T_TEXT_MAIN_SCREEN_TIME_SYSTEM;
    if ( format12 )    //  12-Hour format
        gTextTextures[ index ]->set_position( 260, 365 );
    else                //  24-Hour format
        gTextTextures[ index ]->set_position( 280, 360 );

    /*  Position the alarm time text */
    index = T_TEXT_MAIN_SCREEN_TIME_ALARM;
    if ( format12 )    //  12-Hour format
        gTextTextures[ index ]->set_position( 260, 450 );
    else                //  24-Hour format
        gTextTextures[ index ]->set_position( 280, 445 );

    /*  Position the alarm time button */
    gAlarmButton->set_position(
            ( (SCREEN_WIDTH) - gAlarmButton->get_width() ) / 2,
            ( (SCREEN_HEIGHT) - gAlarmButton->get_height() ) - 10 );

    /*  Set position of system time label */
    index = T_TEXT_MAIN_SCREEN_HINT_SYSTEM;
    gTextTextures[ index ]->set_position(
            ( (SCREEN_WIDTH) - ( gTextTextures[ index ]->get_width() )) /2,
            342 );

    /*  Set position of alarm time label */
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT_ALARM ]->set_position(
            gAlarmButton->get_pos_x() + ( gAlarmButton->get_width() / 2 ) -
            ( gTextTextures[ T_TEXT_MAIN_SCREEN_HINT_ALARM ]->get_width() / 2),
            gAlarmButton->get_pos_y() - 18 );


    /*  Set position of all the various text labels on the main screen */
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT1 ]->set_position( 190, 120 );
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT2 ]->set_position( 190, 150 );
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT3 ]->set_position( 190, 180 );
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT4 ]->set_position( 190, 210 );
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT5 ]->set_position( 190, 240 );

    /*  Position the system time outline rect */
    index = T_TEXT_MAIN_SCREEN_TIME_SYSTEM;
    sysTimeRect.x = 240;
    sysTimeRect.y = 360;
    sysTimeRect.w = 320;
    sysTimeRect.h = 65;

    /*  Position the alarm time outline rect */
    index = T_TEXT_MAIN_SCREEN_TIME_ALARM;
    alarmTimeRect.x = 240;
    alarmTimeRect.y = (SCREEN_HEIGHT) - gTextTextures[ index ]->get_height()-10;
    alarmTimeRect.w = 320;
    alarmTimeRect.h = 65;
    gAlarmButton->init_draw_rect();

    /*  Position the tooltips */
    for( int i = 0; i < TOTAL_TOOLTIPS; ++i )
    {
        gToolTips[ i ]->set_position( 0, 0 );
    }
}



/*
--------------------------------------------------------------------------------
                               LAYOUT_HELP_SCREEN
--------------------------------------------------------------------------------
*/
extern void layout_help_screen( void )
{
    /*  Position all of the help screen hints */
    gTextTextures[ T_TEXT_HELP_HINT1 ]->set_position( 190, 20 );    //  title
    gTextTextures[ T_TEXT_HELP_HINT2 ]->set_position( 240, 50 );    //  presets
    gTextTextures[ T_TEXT_HELP_HINT3 ]->set_position( 240, 80 );    //  mods

    gTextTextures[ T_TEXT_HELP_HINT4 ]->set_position( 240, 130 );   //  main
    gTextTextures[ T_TEXT_HELP_HINT5 ]->set_position( 240, 160 );   //  help
    gTextTextures[ T_TEXT_HELP_HINT6 ]->set_position( 240, 190 );   //  about
    gTextTextures[ T_TEXT_HELP_HINT7 ]->set_position( 240, 220 );   //  format

    gTextTextures[ T_TEXT_HELP_HINT8 ]->set_position( 240, 270 );   //  go
    gTextTextures[ T_TEXT_HELP_HINT9 ]->set_position( 240, 300 );   //  quit
}



/*
--------------------------------------------------------------------------------
                              LAYOUT_ABOUT_SCREEN
--------------------------------------------------------------------------------
*/
extern void layout_about_screen( void )
{
    /*  Position the about screen text hints */
    gTextTextures[ T_TEXT_ABOUT_HINT1 ]->set_position( 175, 40 );
    gTextTextures[ T_TEXT_ABOUT_HINT2 ]->set_position( 175, 70 );
    gTextTextures[ T_TEXT_ABOUT_HINT3 ]->set_position( 175, 100 );

    gTextTextures[ T_TEXT_ABOUT_HINT4 ]->set_position( 175, 160 );
    gTextTextures[ T_TEXT_ABOUT_HINT5 ]->set_position( 175, 220 );
    gTextTextures[ T_TEXT_ABOUT_HINT6 ]->set_position( 265, 250 );
}



/*
--------------------------------------------------------------------------------
                              LAYOUT_ALARM_SCREEN
--------------------------------------------------------------------------------
*/
extern void layout_alarm_screen( void )
{
    /*  Index of the texture */
    int i;

    /*  Current system time label */
    i = T_TEXT_ALARM_SCREEN_HINT1;
    gTextTextures[ i ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ i ]->get_width() ) / 2,
            150 );

    /*  Current system time */
    i = T_TEXT_ALARM_SCREEN_SYSTEM;
    gTextTextures[ i ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ i ]->get_width() ) / 2,
            190 );

    /*  Current alarm time label */
    i = T_TEXT_ALARM_SCREEN_HINT2;
    gTextTextures[ i ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ i ]->get_width() ) / 2,
            300 );

    /*  Current alarm time */
    i = T_TEXT_ALARM_SCREEN_CURRENT;
    gTextTextures[ i ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ i ]->get_width() ) / 2,
            330 );

    /*  Bottom of the screen hint */
    gTextTextures[ T_TEXT_ALARM_SCREEN_HINT3 ]->set_position(
            ( (SCREEN_WIDTH) - gTextTextures[ T_TEXT_ALARM_SCREEN_HINT3 ]->
              get_width() ) / 2, (SCREEN_HEIGHT) - 30 );
}
