#ifndef E_UTIL_H
#include "eutil.h"
#endif



/*
--------------------------------------------------------------------------------
                                  CONSTRUCTOR
--------------------------------------------------------------------------------
*/
extern EPresetButton::EPresetButton( void )
{
    /*  Initial padding */
    ePadding.x = 0;
    ePadding.y = 0;

    /*  Init highlight */
    eHighlight = false;
}


/*
--------------------------------------------------------------------------------
                                     RENDER
--------------------------------------------------------------------------------
*/
extern void EPresetButton::render( void )
{
    /*  Draw a rectangle around the button */
    SDL_RenderDrawRect( gRenderer, &eDrawRect );

    /*  If activated, render filled rect */
    if( eActivated == true )
    {
        SDL_RenderFillRect( gRenderer, &eDrawRect );
    }

    /*  Fill the button with dark grey */
    else if( eHighlight == true )
    {
        SDL_SetRenderDrawColor( gRenderer, 32, 32, 32, 0xff );
        SDL_RenderFillRect( gRenderer, &eDrawRect );
        SDL_SetRenderDrawColor( gRenderer, 0xff, 0xff, 0xff, 0xff );
    }

    /*  Render the texture */
    gTextTextures[ eTextureIndex ]->render();
}


/*
--------------------------------------------------------------------------------
                                HANDLE_HIGHLIGHT
--------------------------------------------------------------------------------
*/
extern void EPresetButton::handle_highlight( void )
{
    /*  If the mouse is inside the button, highlight it */
    if( inside_button() )
        eHighlight = true;
    else
        eHighlight = false;
}


/*
--------------------------------------------------------------------------------
                                 INIT_DRAW_RECT

    Init the draw rect to whatever was passed, or to default values if nothing
    was passed.
--------------------------------------------------------------------------------
*/
extern void EPresetButton::init_draw_rect( SDL_Rect *rect )
{
    if( rect == NULL )
    {
        eDrawRect.x = ePos.x;
        eDrawRect.y = ePos.y;
        eDrawRect.w = eWidth;
        eDrawRect.h = eHeight;
    }
    else
    {
        eDrawRect.x = rect->x;
        eDrawRect.y = rect->y;
        eDrawRect.w = rect->w;
        eDrawRect.h = rect->h;
    }
}


/*
--------------------------------------------------------------------------------
                                 SET_HIGHLIGHT
--------------------------------------------------------------------------------
*/
extern void EPresetButton::set_highlight( bool highlight )
{
    eHighlight = highlight;
}




/*
--------------------------------------------------------------------------------
                                  SET_PADDING
--------------------------------------------------------------------------------
*/
extern void EPresetButton::set_padding( int padX, int padY )
{
    /*  Padding */
    ePadding.x = padX;

    /*  If they don't supply a Y padding, use the same padding as the X */
    if( padY < 0 )
        ePadding.y = padX;
    else
        ePadding.y = padY;
}


/*
--------------------------------------------------------------------------------
                                  GET_PADDING
--------------------------------------------------------------------------------
*/
extern SDL_Point EPresetButton::get_padding( void )
{
    return( ePadding );
}


/*
--------------------------------------------------------------------------------
                                 GET_PADDING_X
--------------------------------------------------------------------------------
*/
extern int EPresetButton::get_padding_x( void )
{
    return( ePadding.x );
}


/*
--------------------------------------------------------------------------------
                                 GET_PADDING_Y
--------------------------------------------------------------------------------
*/
extern int EPresetButton::get_padding_y( void )
{
    return( ePadding.y );
}
