/*------------------------------------------------------------------------------
 *  This is the texture class.  Its three main purposes are to create textures
 *  (from image files or from strings of text), to hold that texture data, and
 *  to copy that texture to the global renderer.
 *----------------------------------------------------------------------------*/
#ifndef E_CLASS_TEXTURE_H
#define E_CLASS_TEXTURE_H

/*
 * The texture class
 */
class ETexture
{
    public:
        /*  Constructor */
        ETexture( void );

        /*  Destructor */
        ~ETexture( void );

        /*  Load an image from file */
        bool load_texture_from_file( const char *path );

        /*  Create a texture from given string */
        bool create_texture_from_string( const char *string, TTF_Font *font,
                SDL_Color textColor );

        /*  Deallocate texture data */
        void free_texture( void );

        /*  Render texture to the screen */
        void render( int x = -1, int y = -1 );

        /*  Get texture dimensions */
        int get_width( void );
        int get_height( void );

        /*  Set the texture default position */
        void set_position( int x, int y );

        /*  Get the position */
        int get_pos_x( void );
        int get_pos_y( void );

        /*  Detect if mouse is inside texture position */
        bool is_inside( void );

    private:
        /*  The texture pointer */
        SDL_Texture *eTexture;

        /*  Texture dimensions */
        int eWidth;
        int eHeight;

        /*  Texture default render position */
        int ePosX;
        int ePosY;
};

#endif
