/*------------------------------------------------------------------------------
 *  These are the time objects.  There are 12 in total:  One for each of the ten
 *  presets, one for the system time and one for the alarm time.  Each time
 *  object's primary purpose is to store or modify its own time, keeping track
 *  for whatever button/texture the time object is associated with.
 *----------------------------------------------------------------------------*/
#ifndef E_CLASS_TIME_H
#define E_CLASS_TIME_H

/*
 *  Time class
 */
class ETime
{
    public:
        /*  Constructor */
        ETime( int time = 0 );

        /*  Calculate number of seconds since midnight */
        int seconds_since_midnight( int h, int m, int s );

        /*  Calculate and set the time */
        void calculate_time( time_t *rawTime );

        /*  Translate the time from seconds since midnight to hr / min / sec */
        void translate_time( void );

        /*  Check the hour / min / sec to make sure they're good */
        void correct_time( void );

        /*  Convert the time to a usable string ( 16 characters ) */
        char* time_to_str( char *string );

        /*  Save or restore the time to/from backup */
        void save_time( void );
        void restore_time( void );

        /*  Get or set the time */
        int get_time( void );
        void set_time( int time );

        /*  Set the time from the hour / min / second */
        void set_time_tr( int h = -1, int m = -1, int s = -1 );

        /*  Time setting functions */
        void set_hour( int hour );
        void set_min( int min );
        void set_sec( int sec );

        /*  Time getting functions */
        int get_hour( void );
        int get_min( void );
        int get_sec( void );

        /*  Time ticking methods */
        void tick_hour( int tick );
        void tick_min( int tick );
        void tick_sec( int tick );

    private:
        /*  The assigned time, in seconds since midnight */
        int eTime;

        /*  Previously set time */
        int eSaved;

        /*  Time of day integers */
        int eHour;
        int eMin;
        int eSec;
};

#endif
