/*------------------------------------------------------------------------------
 *  This file is where many of our globals are declared and initialized.
 *----------------------------------------------------------------------------*/
#ifndef E_UTIL_H
#include "eutil.h"
#endif

/*  Our global window */
SDL_Window *gWindow;

/*  Our global renderer */
SDL_Renderer *gRenderer;

/*  A few rectangles */
SDL_Rect presetRect;        //  Rectangle drawn around each preset time
SDL_Rect sysTimeRect;       //  Rectangle drawn around the system time
SDL_Rect alarmTimeRect;     //  Rectangle drawn around the alarm time
SDL_Rect timeSlotRect;      //  Rectangle drawn around the selected time slot

/*  Colors that we'll be using */
SDL_Color textColor = { 255, 255, 255, 255 };           //  White
SDL_Color alarmTextColor = { 0xff, 0x7f, 0, 0xff };     //  Orangey
SDL_Color activateColor = { 255, 255, 0, 255 };         //  Yellow

/*  TTF fonts */
TTF_Font *gFonts[ (TOTAL_FONTS) ];                      //  Normal fonts
TTF_Font *gFontsMono[ (TOTAL_FONTS) ];                  //  Monospace fonts

/*  Textures */
ETexture *gButtonTextures[ (TOTAL_GRAPHICAL_TEXTURES) ];    //  Graphical
ETexture *gTextTextures[ (TOTAL_TEXT_TEXTURES) ];           //  Text
ETexture *gToolTips[ TOTAL_TOOLTIPS ];                      //  Tooltips

/*  Buttons */
EButton *gMainButtons[ T_BUTTON_UP ];                  //  Main screen
EButton *gTimeButtons[ TOTAL_TIME_BUTTONS ];                //  Time screen
EPresetButton *gPresetButtons[ TOTAL_PRESET_BUTTONS ];      //  Presets
EPresetButton *gAlarmButton = new EPresetButton;            //  Alarm time btn

/*  Times - All presets, the system time, the alarm time */
ETime *gTimes[ TOTAL_TIMES ];

/*  Global booleans */
bool format12 = false;  //  Whether or not to use 12-Hour format (default:  no)
bool ring = false;      //  Whether or not the alarm is currently 'ringing'

/*  Global integers */
int currentScreen = 0;                  //  The screen we're on
int changingTime = 0;                   //  Which button is changing the time
int currentToolTip = -1;                //  Which tooltip we're displaying
int changingTimeFocus = FOCUS_HOUR;     //  The 'time slot' selected
Uint32 heldTicks = 0;                   //  How long the user has held a key

/*  Use for system time (time.h) */
time_t sysTimeRaw;
struct tm *sysTime;

/*  Character arrays */
char sysTimeStr[ 16 ];      //  The system time
char soundFile[ 256 ];      //  The sound file we're using
char sysDir[ 256 ];         //  The eAlarm top system directory
char userDir[ 256 ];        //  The user's eAlarm directory
char indexFile[ 256 ];      //  The index file with a list of textures to load
char dataDir[ 256 ];        //  The data directory containing said textures
char configFile[ 256 ];     //  The config file to use




/*
--------------------------------------------------------------------------------
                               CHANGE_TIME_STRING
--------------------------------------------------------------------------------
*/
extern void change_time_string( int id )
{
    /*  String to hold our text */
    char changingTimeStr[ 16 ];

    /*  The font size we want to use */
    int fontSize = SIZE_BIGGEST;

    /*  Get the times */
    int hour = gTimes[ id ]->get_hour();
    int min = gTimes[ id ]->get_min();
    int sec = gTimes[ id ]->get_sec();

    /*  Using 12-hour format */
    if( format12 == true )
    {

        fontSize = SIZE_HUGE;

        /*  Correct the hour */
        if( hour > 12 )
            hour -= 12;

        if( hour == 0 )
            hour = 12;

        /*  If we're at noon or later */
        if( gTimes[ id ]->get_hour() > 11 )
            snprintf( changingTimeStr, 16, "%02d : %02d : %02d PM", hour, min,
                    sec );

        /*  Earlier than noon */
        else
            snprintf( changingTimeStr, 16, "%02d : %02d : %02d AM", hour, min,
                    sec );
    }

    /*  Using 24-hour format */
    else
        snprintf( changingTimeStr, 16, "%02d : %02d : %02d", hour, min, sec );


    /*  Create texture from the string */
    gTextTextures[ T_TEXT_TIME_TIME ]->create_texture_from_string(
            changingTimeStr, gFonts[ fontSize ], textColor );
}



/*
--------------------------------------------------------------------------------
                                INIT_TIME_SCREEN
--------------------------------------------------------------------------------
*/
extern void init_time_screen( int id )
{
    /*  Save the time object's currently set time */
    gTimes[ id ]->save_time();

    /*  Translate the time to something we can use */
    gTimes[ id ]->translate_time();

    /*  Change the time texture to the time located in gTimes[ id ] */
    change_time_string( id );

    /*  Create the texture for our time screen title */
    char timeTitleStr[ 32 ];
    if( id < TIME_ALARM )
        snprintf( timeTitleStr, 32, "Changing alarm preset %02d", id + 1 );
    else
        snprintf( timeTitleStr, 32, "Changing current alarm" );

    /*  Create texture from the text */
    gTextTextures[ T_TEXT_TIME_TITLE ]->create_texture_from_string(timeTitleStr,
            gFonts[ SIZE_BIG ], textColor );

    /*  Reconfigure the time screen layout */
    layout_time_screen();

}


/*
--------------------------------------------------------------------------------
                               INIT_PRESET_TIMES
--------------------------------------------------------------------------------
*/
extern void init_preset_times( void )
{
    /*  String to hold our text */
    char timeStr[ 16 ];

    /*  We need this to change the font size in case of format switching */
    int fontSize;

    /*  Create textures for the preset buttons  + alarm time */
    for( int i = 0; i < TOTAL_PRESET_BUTTONS + 1; ++i )
    {
        /*  Translate the time to what we want */
        gTimes[ i ]->translate_time();

        /*  Create our string */
        gTimes[ i ]->time_to_str( timeStr );

        /*  Adjust the font size as needed */
        if( format12 )
            fontSize = SIZE_SMALL;
        else
            fontSize = SIZE_MEDIUM;

        /*  Create the texture */
        gTextTextures[ i ]->create_texture_from_string( timeStr,
                gFonts[ fontSize ], textColor );
    }
}



/*
--------------------------------------------------------------------------------
                                INIT_ALARM_TIME
--------------------------------------------------------------------------------
*/
extern void init_alarm_time( void )
{
    char timeStr[ 16 ];
    int fontSize;

    /*  Adjust the font size as needed */
    if( format12 )
        fontSize = SIZE_HUGE;
    else
        fontSize = SIZE_BIGGEST;

    /*  Translate the time to something more manageable */
    gTimes[ TIME_ALARM ]->translate_time();

    /*  Put the translated time into timeStr */
    gTimes[ TIME_ALARM ]->time_to_str( timeStr );

    /*  Create texture from timeStr at gTextTextures[ TIME_ALARM ] */
    if( currentScreen == SCREEN_ALARM )
    {
        gTextTextures[ T_TEXT_ALARM_SCREEN_CURRENT ]->
            create_texture_from_string( timeStr, gFonts[ SIZE_HUGE ],
                    alarmTextColor );
    }

    else
    {
        gTextTextures[ TIME_ALARM ]->create_texture_from_string( timeStr,
                gFonts[ fontSize ], alarmTextColor );
    }
}



/*
--------------------------------------------------------------------------------
                               INIT_TIME_BUTTONS
--------------------------------------------------------------------------------
*/
extern void init_time_buttons( void )
{
    init_preset_times();
    init_alarm_time();
}



/*
--------------------------------------------------------------------------------
                               UPDATE_SYSTEM_TIME
--------------------------------------------------------------------------------
*/
extern void update_system_time( void )
{
    int fontSize = SIZE_BIGGEST;

    if( format12 )
        fontSize = SIZE_HUGE;

    /*  Get the raw time */
    time( &sysTimeRaw );

    /*  Create a useable struct out of it */
    sysTime = localtime( &sysTimeRaw );

    /*  Set the time */
    gTimes[ TIME_SYSTEM ]->set_time_tr( sysTime->tm_hour, sysTime->tm_min,
            sysTime->tm_sec );

    /*  Create the system time texture */
    if( currentScreen == SCREEN_ALARM )
    {
        gTextTextures[ T_TEXT_ALARM_SCREEN_SYSTEM ]->
            create_texture_from_string( gTimes[ TIME_SYSTEM ]->time_to_str(
            sysTimeStr ), gFonts[ SIZE_HUGE ], textColor );
    }

    else
    {
        /*  Create system time texture */
        gTextTextures[ T_TEXT_MAIN_SCREEN_TIME_SYSTEM ]->
            create_texture_from_string( gTimes[ TIME_SYSTEM ]->time_to_str(
            sysTimeStr ), gFonts[ fontSize ], textColor );
    }
}



/*
--------------------------------------------------------------------------------
                               INIT_ALARM_SCREEN
--------------------------------------------------------------------------------
*/
extern void init_alarm_screen( void )
{
    /*  Create system time texture */
    gTextTextures[ T_TEXT_ALARM_SCREEN_SYSTEM ]->
        create_texture_from_string( gTimes[ TIME_SYSTEM ]->time_to_str(
        sysTimeStr ), gFonts[ SIZE_BIGGEST ], textColor );

    init_alarm_time();
}



/*
--------------------------------------------------------------------------------
                                FIND_SYSTEM_DIR
--------------------------------------------------------------------------------
*/
extern bool find_system_dir( void )
{
    /*
     *  Look to see if /usr/share/ealarm or /usr/local/share/ealarm exist.  If
     *  so, use the first we find.  If not, use the present working directory.
     *  If that doesn't work, bail out.
     */
    FILE *fp = NULL;

    if( ( fp = fopen( "/usr/share/ealarm", "r" ) ) != NULL )
    {
        strcpy( sysDir, "/usr/share/ealarm" );
        fclose( fp );
        return( true );
    }

    if( ( fp = fopen( "/usr/local/share/ealarm", "r" ) ) != NULL )
    {
        strcpy( sysDir, "/usr/local/share/ealarm" );
        fclose( fp );
        return( true );
    }

    if( ( fp = fopen( ".", "r") ) != NULL )
    {

        printf("WARNING:  Unable to locate ealarm system dir in ");
        printf("/usr/share/ealarm or\n");
        printf("/usr/local/share/ealarm.  Using present working directory as ");
        printf("system directory.\n");

        snprintf( sysDir, 256, "%s", getenv("PWD" ) );
        fclose( fp );
        return( true );
    }
    else
    {
        printf("ERROR:  Cannot locate ealarm system directory, nor open\n");
        printf("present working directory as system dir.  Aborting.\n");
    }

    return( false );
}



/*
--------------------------------------------------------------------------------
                                 FIND_USER_DIR
--------------------------------------------------------------------------------
*/
extern bool find_user_dir( void )
{
    FILE *fp = NULL;
    char uDir[ 256 ];

    /*  See if it's in $HOME/.ealarm */
    snprintf( uDir, 256, "%s/.ealarm", getenv("HOME") );
    if( ( fp = fopen( uDir, "r" ) ) != NULL )
    {
        strncpy( userDir, uDir, 256 );
        fclose( fp );
        return( true );
    }

    /*  See if it's in $XDF_CONFIG_HOME/ealarm (default $HOME/.config/ealarm) */
    if( getenv("XDG_CONFIG_HOME") != NULL )
    {
        snprintf( uDir, 256, "%s/ealarm", getenv("XDG_CONFIG_HOME") );
        if( ( fp = fopen( uDir, "r" ) ) != NULL )
        {
            strncpy( userDir, uDir, 256 );
            fclose( fp );
            return( true );
        }
    }

    /*  Check for $HOME/.config/ealarm again */
    snprintf( uDir, 256, "%s/.config/ealarm", getenv("HOME") );
    if( ( fp = fopen( uDir, "r" ) ) != NULL )
    {
        strncpy( userDir, uDir, 256 );
        fclose( fp );
        return( true );
    }


    return( false );
}



/*
--------------------------------------------------------------------------------
                                CREATE_USER_DIR
--------------------------------------------------------------------------------
*/
extern bool create_user_dir( void )
{
    char uDir[ 256 ];

    /*
     *  If the user directory didn't exist, create it.  Try a couple of default
     *  locations; if those don't work, bail out
     */

    if( getenv("XDG_CONFIG_HOME") != NULL )
    {
        snprintf( uDir, 256, "%s/ealarm", getenv("XDG_CONFIG_HOME") );
        if( mkdir( uDir, 0755 ) == 0 )
        {
            strncpy( userDir, uDir, 256 );
            return( true );
        }
    }

    snprintf( uDir, 256, "%s/.config/ealarm", getenv("HOME") );
    if( mkdir( uDir, 0755 ) == 0 )
    {
        strncpy( userDir, uDir, 256 );
        return( true );
    }

    snprintf( uDir, 256, "%s/.ealarm", getenv("HOME") );
    if( mkdir( uDir, 0755 ) == 0 )
    {
        strncpy( userDir, uDir, 256 );
        return( true );
    }

    return( false );
}



/*
--------------------------------------------------------------------------------
                                FIND_CONFIG_FILE
--------------------------------------------------------------------------------
*/
extern bool find_config_file( void )
{
    /*  First thing's first, we check for the config file env variable */
    if( getenv("EALARM_CONFIG") != NULL )
    {
        strncpy( configFile, getenv("EALARM_CONFIG"), 256 );
        return( true );
    }

    FILE *fp = NULL;
    char cfgPath[ 256 ];
    
    /*  First we try to read from the user's config file */
    snprintf( cfgPath, 256, "%s/config", userDir );
    if( ( fp = fopen( cfgPath, "r" ) ) != NULL )
    {
        strncpy( configFile, cfgPath, 256 );
        fclose( fp );
        return( true );
    }

    /*  If that didn't exist, use the system config file */
    snprintf( cfgPath, 256, "%s/config", sysDir );
    if( ( fp = fopen( cfgPath, "r" ) ) != NULL )
    {
        strncpy( configFile, cfgPath, 256 );
        fclose( fp );
        return( true );
    }

    return( false );
}



/*
--------------------------------------------------------------------------------
                             SET_DEFAULT_SOUND_FILE
--------------------------------------------------------------------------------
*/
extern bool set_default_sound_file( void )
{
    FILE *fp = NULL;
    
    /*  Assuming the user didn't specify a sound file, use the default */
    snprintf( soundFile, 256, "%s/ring.wav", dataDir );

    if( ( fp = fopen( soundFile, "r" ) ) == NULL )
    {
        return( false );
    }
    else
    {
        fclose( fp );
        return( true );
    }
}



/*
--------------------------------------------------------------------------------
                                READ_CONFIG_FILE
--------------------------------------------------------------------------------
*/
extern bool read_config_file( void )
{
    /*  Find the file */
    if( ! find_config_file() )
    {
        printf("WARNING:  Cannot locate config file; using default values\n");
        if( ! set_default_sound_file() )
        {
            printf("ERROR:  Cannot open default sound file\n");
            return( false );
        }

        return( true );
    }

    int hour, min, sec;

    FILE *fp = fopen( configFile, "r" );
    if( fp == NULL )
    {
        printf("WARNING:  Unable to read from config file\n");
        return( true );
    }

    /*  Read from config file */
    if( fp != NULL )
    {
        char line[ 256 ];
        char formatStr[ 64 ];

        int timePref = 0;
        char userSoundFile[ 256 ];

        while( fgets( line, 256, fp ) )
        {
            /*  Look for preset strings */
            for( int i = 0; i < TOTAL_PRESET_BUTTONS; ++i )
            {
                /*  Reset hour/min/sec */
                hour = min = sec = 0;

                snprintf( formatStr, 64,"PRESET_%d=%%d%%*c%%d%%*c%%d", i + 1 );

                /*  Set the given preset time */
                if( sscanf( line, (const char*)formatStr, &hour, &min, &sec ) )
                {
                    gTimes[ i ]->set_time_tr( hour, min, sec );
                }
            }


            /*  Now scan for the current alarm */
            hour = min = sec = 0;
            if( sscanf( line, "CURRENT=%d%*c%d%*c%d", &hour, &min, &sec ) )
            {
                gTimes[ TIME_ALARM ]->set_time_tr( hour, min, sec );
            }

            /*  Scan for time format preference */
            if( sscanf( line, "USE_12H_FORMAT=%d", &timePref ) )
            {
                if( timePref > 0 )
                    format12 = true;
                else
                    format12 = false;
            }

            /*  Scan for user specified sound file */
            if( sscanf( line, "SOUND_FILE=%s", userSoundFile ) )
            {
                /*  If the user doesn't care, just use the default */
                if( strncmp( "default", userSoundFile, 7 ) == 0 ||
                        strncmp( "DEFAULT", userSoundFile, 7 ) == 0 ||
                        strncmp( "0", userSoundFile, 1 ) == 0 )
                {
                    if( ! set_default_sound_file() )
                    {
                        printf("ERROR:  Cannot open default sound file\n");
                        return( false );
                    }
                }

                /*  If the user does care, try to open it for reading */
                else
                {
                    FILE *sf = fopen( userSoundFile, "r" );

                    /*  If it can't be opened, use default sound */
                    if( sf == NULL )
                    {
                        printf("WARNING:  Cannot open '%s' ", userSoundFile );
                        printf("for reading, using default sound file ");
                        printf("instead\n");

                        if( ! set_default_sound_file() )
                        {
                            printf("ERROR:  Cannot open default sound file\n");
                            return( false );
                        }
                    }

                    else
                    {
                        fclose( sf );
                        strncpy( soundFile, userSoundFile, 256 );
                    }
                }
            }
        }
    }

    /*  Close the file */
    fclose( fp );

    return( true );
}



/*
--------------------------------------------------------------------------------
                                  INIT_CONFIG
--------------------------------------------------------------------------------
*/
extern bool init_config( void )
{
    FILE *fp = NULL;

    /*  If we don't find it at first, create the directory */
    if( ! find_user_dir() )
    {
        create_user_dir();

        /*  If we still can't find it */
        if( ! find_user_dir() )
        {
            printf("WARNING:  Cannot find or create user directory\n");
        }
    }

    /*  Check for system dir; program won't run without it */
#ifdef UPREFIX
    if( ( fp = fopen( xstr(UPREFIX), "r") ) != NULL )
    {
        snprintf( sysDir, 256, "%s/share/ealarm", xstr(UPREFIX) );
        fclose( fp );
    }

    else
    {
        if( ! find_system_dir() )
        {
            printf("ERROR:  Cannot find system directory");
            return( false );
        }
    }
#else
    if( ! find_system_dir() )
    {
        printf("ERROR:  Cannot find system directory");
        return( false );
    }
#endif

    /*
     *  Find the data dir
     */

    /*  Check in the user dir first */
    snprintf( dataDir, 256, "%s/data", userDir );
    if( ( fp = fopen( dataDir, "r" ) ) == NULL )
    {
        /*  Otherwise, fall back to the system dir */
        snprintf( dataDir, 256, "%s/data", sysDir );
        if( ( fp = fopen( dataDir, "r" ) ) == NULL )
        {
            printf("ERROR:  Cannot locate data directory\n");
            return( false );
        }
    }

    /*  Check to make sure the file is closed */
    if( fp != NULL )
        fclose( fp );

    /*  Create the index file string */
    snprintf( indexFile, 256, "%s/index", dataDir );

    /*  Read from the config file */
    if( ! read_config_file() )
        return( false );


    /*
     *  Certain environment variables override both the default values and the
     *  values specified in the config file
     */

    /*  The data directory environment variable */
    if( getenv("EALARM_DATA_DIR") != NULL )
        strncpy( dataDir, getenv("EALARM_DATA_DIR"), 256 );

    /*  The config file enviroment variable */
    if( getenv("EALARM_CONFIG") != NULL )
        strncpy( configFile, getenv("EALARM_CONFIG"), 256 );

    /*  The ealarm home environment variable */
    if( getenv("EALARM_HOME") != NULL )
        strncpy( userDir, getenv("EALARM_HOME"), 256 );


    return( true );
}



/*
--------------------------------------------------------------------------------
                               WRITE_CONFIG_FILE
--------------------------------------------------------------------------------
*/
extern bool write_config_file( void )
{
    /*  Open the file */
    FILE *fp = fopen( configFile, "w" );
    if( fp == NULL )
    {
        snprintf( configFile, 256, "%s/config", userDir );
        fp = fopen( configFile, "w" );

        if( fp == NULL )
        {
            printf("ERROR:  Could not open config file '%s' for writing\n",
                    configFile );
            return( false );
        }
    }


    /*  ------------------------    HEADER  ------------------------ */

    fputc( '#', fp );
    for( int i = 0; i < 79; ++i )
        fputc( '-', fp );
    fputc( '\n', fp );

    fprintf( fp, "#\n#\t\tConfig file for eAlarm\n");
    fprintf( fp, "#\t\tGenerated on %02d-%02d-%02d at %02d:%02d:%02d\n#\n#",
            sysTime->tm_year + 1900,
            sysTime->tm_mon + 1,
            sysTime->tm_mday,
            sysTime->tm_hour, sysTime->tm_min, sysTime->tm_sec );

    for( int i = 0; i < 79; ++i )
        fputc( '-', fp );
    fputc( '\n', fp );


    /*  ------------------------    BODY  ------------------------ */

    fprintf(fp,"\n\n##  Presets, 1-10.  Each must be set in HH:MM:SS format\n");
    for( int i = 0; i < TOTAL_PRESET_BUTTONS; ++i )
    {
        fprintf( fp, "PRESET_%d=%02d:%02d:%02d\n", i + 1,
                gTimes[ i ]->get_hour(),
                gTimes[ i ]->get_min(),
                gTimes[ i ]->get_sec() );
    }

    fprintf( fp, "\n##  Current alarm time.  Same format as the presets\n" );
    fprintf( fp, "\nCURRENT=%02d:%02d:%02d\n",
            gTimes[ TIME_ALARM ]->get_hour(),
            gTimes[ TIME_ALARM ]->get_min(),
            gTimes[ TIME_ALARM ]->get_sec() );
    

    fprintf( fp,"\n##  If you want to use 12-Hour ('normal') time, set this\n");
    fprintf( fp, "##  to 1.  Otherwise, set it to 0.\n" );
    if( format12 == true )
        fprintf( fp, "USE_12H_FORMAT=1\n" );
    else
        fprintf( fp, "USE_12H_FORMAT=0\n" );


    fprintf( fp, "\n##  Absolute path to chosen sound file.  Set to DEFAULT\n");
    fprintf( fp, "##  if you don't want to set your own.\n");
    fprintf( fp, "SOUND_FILE=%s\n", soundFile );


    /*  Close the file */
    fclose( fp );

    return( true );
}
