/*==============================================================================
 *  ealarm  |   version 0.92    |   GNU GPL v3      |   2014-05-18
 *  James Hendrie               |   hendrie.james@gmail.com
 *
 *  Description:
 *      A remake of my earlier Python/Pygame ealarm program.  It's a basic
 *      alarm clock, this time made with C++ / SDL2.
 *============================================================================*/
#ifndef E_UTIL_H
#include "eutil.h"
#endif



/*  The ring effect */
Mix_Chunk *gRing = NULL;


/*  Times */
extern ETime *gTimes[];         //  All of the times (presets, system and alarm)

/*  Locally declared integers */
int flashTicks = 0;                 //  Used to time the flashes during ringing
int now;                            //  System time, in seconds since midnight
int set;                            //  Alarm time, in seconds since midnight

/*  Local booleans */
bool flashing;                      //  Whether or not we're flashing
bool quitting = false;              //  Whether or not we're quitting
bool testing = false;               //  Whether or not we're testing the alarm
bool argToTime = false;             //  Starting with a time given as an arg

/*  The frames per second */
const int FPS = 30;                 //  Crude FPS setting ( delay:  1000 / FPS )




/*
--------------------------------------------------------------------------------
                                      INIT
--------------------------------------------------------------------------------
*/
bool init( void )
{

    /*  Init SDL */
    if( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_AUDIO ) < 0 )
    {
        printf("Unable to init SDL!  SDL Error:  %s\n", SDL_GetError() );
        return( false );
    }

    /*  Init SDL_image */
    if( ( IMG_Init( IMG_INIT_PNG ) & IMG_INIT_PNG ) == false )
    {
        printf("Unable to init SDL_image!  SDL_image Error:  %s\n",
                IMG_GetError() );
        return( false );
    }

    /*  Init SDL_mixer */
    if( Mix_OpenAudio( 44100, MIX_DEFAULT_FORMAT, 2, 2048 ) < 0 )
    {
        printf("Unable to init SDL_mixer!  SDL_mixer Error:  %s\n",
                Mix_GetError() );
        return( false );
    }

    /*  Init SDL_ttf */
    if( TTF_Init() == - 1 )
    {
        printf("Unable to init SDL_ttf!  SDL_ttf Error:  %s\n",
                TTF_GetError() );
        return( false );
    }

    /*  Create our window */
    gWindow = SDL_CreateWindow( "eAlarm",
            SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
            (SCREEN_WIDTH), (SCREEN_HEIGHT), SDL_WINDOW_SHOWN );
    if( gWindow == NULL )
    {
        printf("Unable to create window!  SDL Error:  %s\n", SDL_GetError() );
        return( false );
    }


    /*  Create our renderer */
    gRenderer = SDL_CreateRenderer( gWindow, -1, SDL_RENDERER_ACCELERATED );

    if( gRenderer == NULL )
    {
        printf("Unable to create renderer!  SDL Error:  %s\n",
                SDL_GetError() );
        return( false );
    }


    return( true );
}



/*
--------------------------------------------------------------------------------
                              LOAD_BUTTON_TEXTURES
--------------------------------------------------------------------------------
*/
bool load_button_textures( void )
{
    /*
     * I'm lazy, so we're just going to read out a list of files to load and
     * load them
     */
    FILE *fp = fopen( indexFile, "r" );
    if( fp == NULL )
    {
        printf("ERROR:  Could not open the index file.\n");
        return( false );
    }

    /*
     *  Each file name to load.  I know none of the file names are 256 chars,
     *  but who knows, maybe people can make their own lists of files for
     *  theming purposes or whatever
     */
    char line[ 256 ];
    char fileName[ 256 ];
    int currentIndex = 0;
    while( fgets( line, 256, fp ) )
    {
        /*  Cut off the newline character at the end */
        line[ strlen( line ) - 1 ] = '\0';

        /*  Create the file name */
        snprintf( fileName, 256, "%s/%s", dataDir, line );

        /*  Create the texture */
        gButtonTextures[ currentIndex ] = new ETexture();

        /*  Load the texture */
        if( gButtonTextures[ currentIndex ]->load_texture_from_file(
                    fileName ) == false )
        {
            printf("Unable to load file %s\n", fileName );
            return( false );
        }

        /*  Increment the index */
        ++currentIndex;
    }

    /*  Close the file */
    fclose( fp );
    fp = NULL;

    return( true );
}




/*
--------------------------------------------------------------------------------
                               LOAD_TEXT_TEXTURES
--------------------------------------------------------------------------------
*/
bool load_text_textures( void )
{
    /*  Create every text instance you'll need */
    for( int i = 0; i < TOTAL_TEXT_TEXTURES; ++i )
    {
        gTextTextures[ i ] = new ETexture();
    }

    /*  Create all of the tooltip textures */
    for( int i = 0; i < TOTAL_TOOLTIPS; ++i )
    {
        gToolTips[ i ] = new ETexture();
    }

    /*  --------------  MAIN SCREEN TEXT    ---------------------- */

    /*  Current time texture */
    gTextTextures[ T_TEXT_MAIN_SCREEN_TIME_ALARM]->create_texture_from_string(
            "12:00:00", gFonts[ SIZE_BIGGEST ], alarmTextColor );

    /*  The main screen text 'hints' */
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT1 ]->create_texture_from_string(
            "Click on a preset to use it as the current",
            gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT2 ]->create_texture_from_string(
            "alarm.  Right-Click (or Shift-Click) to edit",
            gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT3 ]->create_texture_from_string(
            "a time.  Click on the 'Help' button for",
            gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT4 ]->create_texture_from_string(
            "additional help, or the 'About' button for",
            gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT5 ]->create_texture_from_string(
            "more information about this application.",
            gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT_SYSTEM ]->create_texture_from_string(
            "SYSTEM TIME", gFonts[ SIZE_TINY ], textColor );

    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT_ALARM ]->create_texture_from_string(
            "ALARM TIME", gFonts[ SIZE_TINY ], textColor );


    /*  --------------  HELP SCREEN TEXT    ---------------------- */

    gTextTextures[ T_TEXT_HELP_HINT1 ]->create_texture_from_string(
            "Keyboard shortcuts", gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_HELP_HINT2 ]->create_texture_from_string(
            "1-0:        Set alarm to preset N", gFonts[ SIZE_SMALL ],
            textColor );

    gTextTextures[ T_TEXT_HELP_HINT3 ]->create_texture_from_string(
            "Shift+1-0:  Edit preset N", gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_HELP_HINT4 ]->create_texture_from_string(
            "M:          Main screen", gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_HELP_HINT5 ]->create_texture_from_string(
            "H:          Help screen", gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_HELP_HINT6 ]->create_texture_from_string(
            "A:          About screen", gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_HELP_HINT7 ]->create_texture_from_string(
            "F:          Switch format ( 24h / 12h )", gFonts[ SIZE_SMALL ],
            textColor );

    gTextTextures[ T_TEXT_HELP_HINT8 ]->create_texture_from_string(
            "Enter:      Activate alarm", gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_HELP_HINT9 ]->create_texture_from_string(
            "ESC or Q:   Quit application", gFonts[ SIZE_SMALL ], textColor );


    /*  --------------  ABOUT SCREEN TEXT    ---------------------- */

    gTextTextures[ T_TEXT_ABOUT_HINT1 ]->create_texture_from_string(
            "This program functions as a basic alarm clock.",
            gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_ABOUT_HINT2 ]->create_texture_from_string(
            "It was written as a bit of a training exercise",
            gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_ABOUT_HINT3 ]->create_texture_from_string(
            "in SDL2.", gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_ABOUT_HINT4 ]->create_texture_from_string(
            "This program is licensed under the GPLv3.",
            gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_ABOUT_HINT5 ]->create_texture_from_string(
            "Author:  James Hendrie", gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_ABOUT_HINT6 ]->create_texture_from_string(
            "<hendrie.james@gmail.com>", gFonts[ SIZE_SMALL ], textColor );



    /*  --------------  TOOLTIPS    ---------------------- */

    gToolTips[ TOOLTIP_MAIN ]->create_texture_from_string(
            "Click to view the main screen", gFonts[ SIZE_TINY ], textColor );

    gToolTips[ TOOLTIP_HELP ]->create_texture_from_string(
            "Click to view keyboard shortcuts",
            gFonts[ SIZE_TINY ], textColor );

    gToolTips[ TOOLTIP_ABOUT ]->create_texture_from_string(
            "Click to view addition program information",
            gFonts[ SIZE_TINY ], textColor );

    gToolTips[ TOOLTIP_FORMAT ]->create_texture_from_string(
            "Click to switch between 24-Hour and 12-Hour time format",
            gFonts[ SIZE_TINY ], textColor );

    gToolTips[ TOOLTIP_GO ]->create_texture_from_string(
            "Click to activate the alarm", gFonts[ SIZE_TINY ], activateColor );

    gToolTips[ TOOLTIP_PRESET ]->create_texture_from_string(
            "LMB:  Set alarm to preset time.  RMB:  Edit preset time",
            gFonts[ SIZE_TINY ], textColor );

    gToolTips[ TOOLTIP_SYSTEM ]->create_texture_from_string(
            "The current system time", gFonts[ SIZE_TINY ], textColor );

    gToolTips[ TOOLTIP_ALARM ]->create_texture_from_string(
            "Time at which the alarm will ring", gFonts[ SIZE_TINY ],
            textColor );


    /*  --------------  ALARM SCREEN TEXT    ---------------------- */

    gTextTextures[ T_TEXT_ALARM_SCREEN_HINT1 ]->create_texture_from_string(
            "CURRENT SYSTEM TIME:", gFontsMono[ SIZE_BIG ], textColor );

    gTextTextures[ T_TEXT_ALARM_SCREEN_HINT2 ]->create_texture_from_string(
            "ALARM IS SET FOR:", gFontsMono[ SIZE_MEDIUM ], textColor );

    gTextTextures[ T_TEXT_ALARM_SCREEN_HINT3 ]->create_texture_from_string(
            "SPACE or RETURN to return to the main menu.  ESC to quit.",
            gFonts[ SIZE_SMALL ], textColor );


    /*  --------------  TIME SCREEN TEXT    ---------------------- */
    gTextTextures[ T_TEXT_TIME_TIME ]->create_texture_from_string(
            "12 : 00 : 00", gFonts[ SIZE_BIGGEST ], textColor );

    gTextTextures[ T_TEXT_TIME_TITLE ]->create_texture_from_string(
            "Changing time", gFonts[ SIZE_BIG ], textColor );

    gTextTextures[ T_TEXT_TIME_HINT1 ]->create_texture_from_string(
            "To increase by 1, use LMB or the UP/DOWN arrow keys",
            gFonts[ SIZE_SMALL ], textColor);

    gTextTextures[ T_TEXT_TIME_HINT2 ]->create_texture_from_string(
            "To increase by 3 (hour) or 5 (min/sec), hold CTRL or use RMB",
            gFonts[ SIZE_SMALL ], textColor );

    gTextTextures[ T_TEXT_TIME_HINT3 ]->create_texture_from_string(
            "To increase by 6 (hour) or 15 (min/sec), hold SHIFT",
            gFonts[ SIZE_SMALL ], textColor );



    return( true );
}



/*
--------------------------------------------------------------------------------
                                   LOAD MEDIA
--------------------------------------------------------------------------------
*/
bool load_media( void )
{
    /*  We'll increase this as we go along */
    int fontSize = 16;
    char fontStr[ 256 ];

    /*  Load our fonts */
    for( int i = 0; i < (TOTAL_FONTS); ++i )
    {
        /*  Load normal font */
        snprintf( fontStr, 256, "%s/Ubuntu-L.ttf", dataDir );
        gFonts[ i ] = TTF_OpenFont( fontStr, fontSize );
        if( gFonts[ i ] == NULL )
        {
            printf("Failed to load font '%s'.  SDL_ttf Error:  %s\n",
                    "data/Ubuntu-R.ttf", TTF_GetError() );
            return( false );
        }

        /*  Load monospace font */
        snprintf( fontStr, 256, "%s/UbuntuMono-R.ttf", dataDir );
        gFontsMono[ i ] = TTF_OpenFont( fontStr, fontSize );
        if( gFontsMono[ i ] == NULL )
        {
            printf("Failed to load font '%s'.  SDL_ttf Error:  %s\n",
                    "data/UbuntuMono-R.ttf", TTF_GetError() );
            return( false );
        }

        /*  Increase our font size */
        fontSize += 8;
    }

    /*  Load our ring sound effect */
    gRing = Mix_LoadWAV( soundFile );
    if( gRing == NULL )
    {
        printf("Failed to load ring sound!  SDL_mixer Error:  %s\n",
                Mix_GetError() );
        return( false );
    }

    /*  Load all of the button textures */
    if( load_button_textures() == false )
    {
        printf("Unable to load button textures\n");
        return( false );
    }

    /*  Load all of the text textures */
    if( load_text_textures() == false )
    {
        printf("Unable to load text textures\n");
        return( false );
    }


    /*  If we made it this far, awesome */
    return( true );
}



/*
--------------------------------------------------------------------------------
                                  INIT_BUTTONS
--------------------------------------------------------------------------------
*/
void init_buttons( void )
{
    /*
     *  INIT PRESET BUTTONS
     */

    /*  Padding between the outline of the button and its texture */
    int pButtonPadding = 32;

    for( int i = 0; i < TOTAL_PRESET_BUTTONS; ++i )
    {
        /*  Create the button */
        gPresetButtons[ i ] = new EPresetButton();

        /*  Set the padding */
        gPresetButtons[ i ]->set_padding( pButtonPadding, pButtonPadding/8 );

        /*  Set the button type */
        gPresetButtons[ i ]->set_type( BT_PRESET );

        /*  Set the button preset ID */
        gPresetButtons[ i ]->set_id( i );
    }

    /*  Init the alarm text button */
    gAlarmButton->set_type( BT_PRESET );
    gAlarmButton->set_id( TOTAL_PRESET_BUTTONS );
    gAlarmButton->set_texture_index( T_TEXT_MAIN_SCREEN_TIME_ALARM );
    gAlarmButton->set_width( 320 );
    gAlarmButton->set_height( 65 );

    gAlarmButton->set_padding( 80, 15 );


    /*
     *  INIT MAIN SCREEN BUTTONS
     */
    int currentTextureIndex = 0;
    for( int i = 0; i < T_BUTTON_UP; ++i )
    {
        /*  Set the index of the texture we want */
        currentTextureIndex = i * 2;

        /*  Create button */
        gMainButtons[ i ] = new EButton( );

        /*  Init the width, height and current texture index */
        int width = gButtonTextures[ currentTextureIndex ]->get_width();
        int height = gButtonTextures[ currentTextureIndex ]->get_height();
        gMainButtons[ i ]->init( 0, 0, width, height, currentTextureIndex );
    }

    /*
     *  INIT TIME SCREEN BUTTONS
     */
    for( int i = 0; i < TOTAL_TIME_BUTTONS; ++i )
    {
        gTimeButtons[ i ] = new EButton();
    }

    /*  Set button type for up/down arrows, start at BT_UP_HOUR */
    int buttonType = BT_UP_HOUR;

    /*  Up buttons */
    for( int i = T_BUTTON_UP1; i < T_BUTTON_UP1 + 3; ++i )
    {
        /*  Set the width, height and texture index */
        gTimeButtons[ i ]->init( 0, 0, 
                gButtonTextures[ T_GRAPHICAL_UP_ACTIVATED ]->get_width(),
                gButtonTextures[ T_GRAPHICAL_UP_ACTIVATED ]->get_height(),
                T_GRAPHICAL_UP_ACTIVATED );
        gTimeButtons[ i ]->set_type( buttonType );
        ++buttonType;
    }

    /*  Down buttons */
    for( int i = T_BUTTON_DOWN1; i < T_BUTTON_DOWN1 + 3; ++i )
    {
        /*  Set the width, height and texture index */
        gTimeButtons[ i ]->init( 0, 0,
                gButtonTextures[ T_GRAPHICAL_DOWN_ACTIVATED ]->get_width(),
                gButtonTextures[ T_GRAPHICAL_DOWN_ACTIVATED ]->get_height(),
                T_GRAPHICAL_DOWN_ACTIVATED );
        gTimeButtons[ i ]->set_type( buttonType );
        ++buttonType;
    }

    /*  Okay button */
    gTimeButtons[ T_BUTTON_YES1 ]->init( 0, 0,
            gButtonTextures[ T_GRAPHICAL_YES_ACTIVATED ]->get_width(),
            gButtonTextures[ T_GRAPHICAL_YES_ACTIVATED ]->get_height(),
            T_GRAPHICAL_YES_ACTIVATED );
    gTimeButtons[ T_BUTTON_YES1 ]->set_type( BT_YES );

    /*  Cancel button */
    gTimeButtons[ T_BUTTON_NO1 ]->init( 0, 0,
            gButtonTextures[ T_GRAPHICAL_NO_ACTIVATED ]->get_width(),
            gButtonTextures[ T_GRAPHICAL_NO_ACTIVATED ]->get_height(),
            T_GRAPHICAL_NO_ACTIVATED );
    gTimeButtons[ T_BUTTON_NO1 ]->set_type( BT_NO );


    /*
     * Main screen buttons
     */
    gMainButtons[ T_BUTTON_MAIN ]->set_type( BT_MAIN );         //  MAIN
    gMainButtons[ T_BUTTON_HELP ]->set_type( BT_HELP );         //  HELP
    gMainButtons[ T_BUTTON_ABOUT ]->set_type( BT_ABOUT );       //  ABOUT
    gMainButtons[ T_BUTTON_FORMAT ]->set_type( BT_FORMAT );     //  FORMAT
    gMainButtons[ T_BUTTON_GO ]->set_type( BT_GO );             //  GO

}



/*
--------------------------------------------------------------------------------
                                     CLOSE
--------------------------------------------------------------------------------
*/
void close( void )
{
    /*  Save changes to the config file */
    write_config_file();

    /*  Close all of the global fonts */
    for( int i = 0; i < (TOTAL_FONTS); ++i )
    {
        /*  Close the fonts */
        TTF_CloseFont( gFonts[ i ] );
        TTF_CloseFont( gFontsMono[ i ] );

        /*  Null the pointers out */
        gFonts[ i ] = NULL;
        gFontsMono[ i ] = NULL;
    }

    /*  Free all of the graphical textures */
    for( int i = 0; i < (TOTAL_GRAPHICAL_TEXTURES); ++i )
    {
        delete gButtonTextures[ i ];
    }

    /*  Free all of the textual textures */
    for( int i = 0; i < TOTAL_TEXT_TEXTURES; ++i )
    {
        delete gTextTextures[ i ];
    }

    /*  Free all of the tooltip textures */
    for( int i = 0; i < TOTAL_TOOLTIPS; ++i )
    {
        delete gToolTips[ i ];
    }

    /*  Check to make sure the sound chunk isn't going off */
    if( Mix_Playing(-1) > 0 )
        Mix_HaltChannel(-1);

    /*  Free the sound chunk */
    Mix_FreeChunk( gRing );
    gRing = NULL;

    /*  Destroy the renderer and window */
    SDL_DestroyRenderer( gRenderer );
    SDL_DestroyWindow( gWindow );
    gRenderer = NULL;
    gWindow = NULL;

    /*  Quit SDL and its subsystems */
    Mix_Quit();
    TTF_Quit();
    IMG_Quit();
    SDL_Quit();
}



/*
--------------------------------------------------------------------------------
                             CHANGE_TO_TIME_SCREEN
--------------------------------------------------------------------------------
*/
void change_to_time_screen( int id )
{
    /*  Just putting this here to make the file less voluminous */
    init_time_screen( id );
    changingTime = id;
    currentScreen = SCREEN_TIME;
}



/*
--------------------------------------------------------------------------------
                               HANDLE_MAIN_SCREEN
--------------------------------------------------------------------------------
*/
void handle_main_screen( SDL_Event &e )
{
    if( e.type == SDL_MOUSEBUTTONDOWN || e.type == SDL_MOUSEBUTTONUP )
    {
        /*  Handle the preset buttons */
        for( int i = 0; i < TOTAL_PRESET_BUTTONS; ++i )
        {
            gPresetButtons[ i ]->handle_event( e );
        }

        /*  Handle the main screen graphical buttons */
        for( int i = 0; i < T_BUTTON_UP; ++i )
        {
            gMainButtons[ i ]->handle_event( e );
        }

        /*  Handle alarm button event */
        gAlarmButton->handle_event( e );
    }

    else if( e.type == SDL_MOUSEMOTION )
    {
        /*  Highlight the preset buttons, if applicable */
        for( int i = 0; i < TOTAL_PRESET_BUTTONS; ++i )
        {
            /*  Handle highlights */
            gPresetButtons[ i ]->handle_highlight();

            /*  Handle tooltips */
            gPresetButtons[ i ]->handle_tooltip();
        }

        /*  Highlight the alarm button, if applicable */
        gAlarmButton->handle_highlight();

        /*  Handle tooltip for alarm */
        gAlarmButton->handle_tooltip();

        /*  Handle tooltip for all the other graphical buttons */
        for( int i = 0; i < T_BUTTON_UP; ++i )
        {
            gMainButtons[ i ]->handle_tooltip();
        }

        /*  Handle tooltip for system time */
        if( gTextTextures[ T_TEXT_MAIN_SCREEN_TIME_SYSTEM ]->is_inside() )
        {
            currentToolTip = TOOLTIP_SYSTEM;
        }
    }

    else if( e.type == SDL_KEYDOWN )
    {
        /*  Init the keysym offset */
        int keySymOffset = SDLK_1 - PRESET1;
        if( e.key.keysym.sym > SDLK_0 && e.key.keysym.sym <= SDLK_9 )
        {
            /*  Edit preset N if they're holding CTRL or SHIFT */
            if( ( SDL_GetModState() & KMOD_SHIFT ) ||
                    ( SDL_GetModState() & KMOD_CTRL ) )
            {
                /*  Edit the time */
                change_to_time_screen( e.key.keysym.sym - keySymOffset );
            }

            /*  Change the current alarm to preset N */
            else
            {
                /*  Set the time */
                gTimes[ TIME_ALARM ]->set_time( gTimes[ 
                    e.key.keysym.sym - keySymOffset ]-> get_time() );

                /*  Highlight the button */
                gPresetButtons[
                    e.key.keysym.sym - keySymOffset ]->set_highlight( true );

                /*  Re-initialize the alarm time */
                init_alarm_time();
            }
        }

        switch( e.key.keysym.sym )
        {
            case SDLK_ESCAPE:       //  ESCAPE key      ( quit )
                quitting = true;
                break;

            case SDLK_m:            //  M key           ( main screen )
                gMainButtons[ T_BUTTON_MAIN ]->activate();
                break;

            case SDLK_h:            //  H key           ( help screen )
                gMainButtons[ T_BUTTON_HELP ]->activate();
                break;

            case SDLK_a:            //  A key           ( about screen )
                gMainButtons[ T_BUTTON_ABOUT ]->activate();
                break;

            case SDLK_f:            //  F key           ( toggle format )
                gMainButtons[ T_BUTTON_FORMAT ]->activate();
                break;

            case SDLK_t:            //  T key           ( test alarm )
                testing = true;
                break;

            case SDLK_RETURN:       //  RETURN key      ( activate alarm )
                gMainButtons[ T_BUTTON_GO ]->activate();
                heldTicks = 0;
                break;

            case SDLK_0:            //  0 key           ( preset 10 )
                /*  Edit preset 10 if they're holding CTRL or SHIFT */
                if( ( SDL_GetModState() & KMOD_SHIFT ) ||
                        ( SDL_GetModState() & KMOD_CTRL ) )
                    change_to_time_screen( PRESET10 );

                /*  Change the current alarm to preset 10 */
                else
                {
                    gTimes[ TIME_ALARM ]->set_time( gTimes[ PRESET10 ]->
                            get_time() );

                    init_alarm_time();
                }
                break;


        }
    }

    else if( e.type == SDL_KEYUP )
    {
        for( int i = 0; i < TOTAL_PRESET_BUTTONS; ++i )
        {
            gPresetButtons[ i ]->set_highlight( false );
        }

        switch( e.key.keysym.sym )
        {
            case SDLK_m:            //  M key       ( main screen )
                gMainButtons[ T_BUTTON_MAIN ]->deactivate();
                currentScreen = SCREEN_MAIN;
                break;

            case SDLK_h:            //  H key       ( help screen )
                gMainButtons[ T_BUTTON_HELP ]->deactivate();
                currentScreen = SCREEN_HELP;
                break;

            case SDLK_a:            //  A key       ( about screen )
                gMainButtons[ T_BUTTON_ABOUT ]->deactivate();
                currentScreen = SCREEN_ABOUT;
                break;

            case SDLK_f:            //  F key       ( toggle format )

                gMainButtons[ T_BUTTON_FORMAT ]->deactivate();

                /*  Toggle format */
                if( format12 == true )
                    format12 = false;
                else
                    format12 = true;

                init_time_buttons();
                layout_main_screen();
                break;

            case SDLK_RETURN:   //  RETURN key      ( switch to alarm screen )
                /*
                 *  heldTicks is the number of 'ticks' that the user has been
                 *  holding down a key for.  If it's more than zero, the alarm
                 *  screen won't be switch to.  We do this to allow the button
                 *  to switch to its deactivated texture, and to keep the
                 *  program from immediately switching back to the alarm screen
                 *  when returning from it.
                 */
                if( heldTicks == 0 )
                {
                    gMainButtons[ T_BUTTON_GO ]->deactivate();
                    currentScreen = SCREEN_ALARM;
                    init_alarm_time();
                    update_system_time();
                    layout_alarm_screen();
                    ++heldTicks;
                }
                break;

            case SDLK_c:        //  C key           ( change current time )
                init_time_screen( TIME_ALARM );
                changingTime = TIME_ALARM;
                currentScreen = SCREEN_TIME;
                break;
        }
    }

}



/*
--------------------------------------------------------------------------------
                                   STOP_ALARM
--------------------------------------------------------------------------------
*/
void stop_alarm( void )
{
    if( Mix_Playing(-1) > 0 )
        Mix_HaltChannel(-1);
}



/*
--------------------------------------------------------------------------------
                              HANDLE_ALARM_SCREEN
--------------------------------------------------------------------------------
*/
void handle_alarm_screen( SDL_Event &e )
{
    /*  NOTE
     *  -----------------------------------------------
     *  Any events we receive here will be KEYDOWN type
     *  -----------------------------------------------
     */

    /*
     * If on the alarm screen, on an SDLK_RETURN or SDLK_SPACE event we want
     * to turn off the alarm and return to the main screen.
     */
    if( e.key.keysym.sym == SDLK_RETURN ||
            e.key.keysym.sym == SDLK_SPACE )
    {
        stop_alarm();
        ring = false;
        currentScreen = SCREEN_MAIN;
        gTimes[ TIME_ALARM ]->restore_time();
        layout_main_screen();
        update_system_time();
    }

    /*  Or, we quit on ESCAPE */
    else if( e.key.keysym.sym == SDLK_ESCAPE )
    {
        stop_alarm();
        quitting = true;
    }
}



/*
--------------------------------------------------------------------------------
                            RENDER_TEXT_TIME_LABELS
--------------------------------------------------------------------------------
*/
void render_text_time_labels( void )
{
    /*  Current system time label */
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT_SYSTEM ]->render();

    /*  Current alarm time label */
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT_ALARM ]->render();
}



/*
--------------------------------------------------------------------------------
                            RENDER_TEXT_MAIN_SCREEN
--------------------------------------------------------------------------------
*/
void render_text_main_screen( void )
{
    /*  Render all of the main screen text hints */
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT1 ]->render();
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT2 ]->render();
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT3 ]->render();
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT4 ]->render();
    gTextTextures[ T_TEXT_MAIN_SCREEN_HINT5 ]->render();

    /*  Render the text time labels */
    render_text_time_labels();

}



/*
--------------------------------------------------------------------------------
                            RENDER_TEXT_HELP_SCREEN
--------------------------------------------------------------------------------
*/
void render_text_help_screen( void )
{
    /*  Render all of the help screen hints */
    gTextTextures[ T_TEXT_HELP_HINT1 ]->render();
    gTextTextures[ T_TEXT_HELP_HINT2 ]->render();
    gTextTextures[ T_TEXT_HELP_HINT3 ]->render();

    gTextTextures[ T_TEXT_HELP_HINT4 ]->render();
    gTextTextures[ T_TEXT_HELP_HINT5 ]->render();
    gTextTextures[ T_TEXT_HELP_HINT6 ]->render();
    gTextTextures[ T_TEXT_HELP_HINT7 ]->render();

    gTextTextures[ T_TEXT_HELP_HINT8 ]->render();
    gTextTextures[ T_TEXT_HELP_HINT9 ]->render();

    /*  Render text time labels */
    render_text_time_labels();
}



/*
--------------------------------------------------------------------------------
                            RENDER_TEXT_ABOUT_SCREEN
--------------------------------------------------------------------------------
*/
void render_text_about_screen( void )
{
    /*  Render all of the about screen text hints */
    gTextTextures[ T_TEXT_ABOUT_HINT1 ]->render();
    gTextTextures[ T_TEXT_ABOUT_HINT2 ]->render();
    gTextTextures[ T_TEXT_ABOUT_HINT3 ]->render();

    gTextTextures[ T_TEXT_ABOUT_HINT4 ]->render();
    gTextTextures[ T_TEXT_ABOUT_HINT5 ]->render();
    gTextTextures[ T_TEXT_ABOUT_HINT6 ]->render();

    /*  Render text time labels */
    render_text_time_labels();
}



/*
--------------------------------------------------------------------------------
                               RENDER_MAIN_SCREEN
--------------------------------------------------------------------------------
*/
void render_main_screen( void )
{
    /*  Render the presets buttons */
    for( int i = 0; i < TOTAL_PRESET_BUTTONS; ++i )
    {
        gPresetButtons[ i ]->render();
    }

    /*  Render the graphical buttons */
    for( int i = 0; i < T_BUTTON_UP; ++i )
    {
        gMainButtons[ i ]->render();
    }

    /*  Update the system time and create its texture */
    update_system_time();

    /*  Render system time texture */
    gTextTextures[ T_TEXT_MAIN_SCREEN_TIME_SYSTEM ]->render();

    /*  Render the system time outline rect */
    SDL_RenderDrawRect( gRenderer, &sysTimeRect );

    /*  Render current alarm time */
    gAlarmButton->render();

    /*  Render the tooltip */
    if( currentToolTip >= 0 )
        gToolTips[ currentToolTip ]->render();
}



/*
--------------------------------------------------------------------------------
                               HANDLE_TIME_SCREEN
--------------------------------------------------------------------------------
*/
void handle_time_screen( SDL_Event &e )
{
    /*  Handle events for all of our 'time screen' buttons */
    for( int i = 0; i < TOTAL_TIME_BUTTONS; ++i )
    {
        gTimeButtons[ i ]->handle_event( e );
    }

    /*
     * If we go left or right, we have to reconfigure the time screen layout
     * since the time slot focus highlight will have changed.
     */
    if( e.key.keysym.sym == SDLK_LEFT || e.key.keysym.sym == SDLK_RIGHT )
    {
        layout_time_screen();
    }

    /*  Change the time string texture */
    change_time_string( changingTime );

}



/*
--------------------------------------------------------------------------------
                               RENDER_TIME_SCREEN
--------------------------------------------------------------------------------
*/
void render_time_screen( void )
{

    /*  Render the title text */
    gTextTextures[ T_TEXT_TIME_TITLE]->render();

    /*  Render the time text */
    gTextTextures[ T_TEXT_TIME_TIME ]->render();

    /*  Render the time screen buttons */
    for( int i = 0; i < TOTAL_TIME_BUTTONS; ++i )
    {
        gTimeButtons[ i ]->render();
    }

    /*  Draw the rect over the currently selected time slot */
    SDL_RenderDrawRect( gRenderer, &timeSlotRect );

    /*  Draw the time screen hints */
    gTextTextures[ T_TEXT_TIME_HINT1 ]->render();
    gTextTextures[ T_TEXT_TIME_HINT2 ]->render();
    gTextTextures[ T_TEXT_TIME_HINT3 ]->render();
}



/*
--------------------------------------------------------------------------------
                               SET_RANDOM_COLORS
--------------------------------------------------------------------------------
*/
void set_random_colors( int *r, int *g, int *b )
{
    /*  Random red color */
    switch( rand() % 3 )
    {
        case 0:
            *r = 0;
            break;
        case 1:
            *r = 127;
            break;
        case 2:
            *r = 255;
            break;
    }

    /*  Random green color */
    switch( rand() % 3 )
    {
        case 0:
            *g = 0;
            break;
        case 1:
            *g = 127;
            break;
        case 2:
            *g = 255;
            break;
    }

    /*  Random blue color */
    switch( rand() % 3 )
    {
        case 0:
            *b = 0;
            break;
        case 1:
            *b = 127;
            break;
        case 2:
            *b = 255;
            break;
    }

}



/*
--------------------------------------------------------------------------------
                              RENDER_ALARM_SCREEN
--------------------------------------------------------------------------------
*/
void render_alarm_screen( void )
{
    /*  Update the system time */
    update_system_time();

    /*  If the screen is allowed to flash, grab random colors and do so */
    if( flashing )
    {
        int r, g, b;
        r = g = b = 0;

        set_random_colors( &r, &g, &b );
        SDL_SetRenderDrawColor( gRenderer, r, g, b, 0xff );

        SDL_RenderClear( gRenderer );

        flashing = false;
    }

    /*  Otherwise, just clear with the normal render color */
    else
    {
        SDL_SetRenderDrawColor( gRenderer, 0, 0, 0, 0xff );
    }

    /*  System time label */
    gTextTextures[ T_TEXT_ALARM_SCREEN_HINT1 ]->render();

    /*  System time */
    gTextTextures[ T_TEXT_ALARM_SCREEN_SYSTEM ]->render();

    /*  Current alarm time label */
    gTextTextures[ T_TEXT_ALARM_SCREEN_HINT2 ]->render();

    /*  Current alarm time */
    gTextTextures[ T_TEXT_ALARM_SCREEN_CURRENT ]->render();


    /*  Bottom screen info */
    gTextTextures[ T_TEXT_ALARM_SCREEN_HINT3 ]->render();

}



/*
--------------------------------------------------------------------------------
                                   INIT_TIMES
--------------------------------------------------------------------------------
*/
void init_times( void )
{
    /*  Init all of our new times and set them to zero */
    for( int i = 0; i < TOTAL_TIMES; ++i )
    {
        gTimes[ i ] = new ETime;

        gTimes[ i ]->set_time( 0 );
    }
}



/*
--------------------------------------------------------------------------------
                                  CHECK_ALARM
--------------------------------------------------------------------------------
*/
void check_alarm( void )
{
    /*
     *  If we're on the alarm screen and the system time matches the alarm time,
     *  it's time to get busy
     */
    if( currentScreen == SCREEN_ALARM )
    {
        if( now == set )
        {
            /*  If the channel isn't already playing, play sound; inf. loop */
            if( Mix_Playing( -1 ) <= 0 )
                Mix_PlayChannel( -1, gRing, -1 );

            ring = true;
        }

        /*  If the alarm is going off */
        if( ring == true )
        {
            /*  We do it this way to allow for a brief pause between flashes */
            ++flashTicks;
            if( flashTicks > 5 )
            {
                flashing = true;
                flashTicks = 0;
            }
        }
    }

}



/*
--------------------------------------------------------------------------------
                                 STRING_TO_TIME
--------------------------------------------------------------------------------
*/
bool string_to_time( const char *string, int id )
{
    /*  Init our time variables */
    int hour, min, sec;
    hour = min = sec = 0;

    /*  Get the length of the string */
    int len = strlen( string );



    /*  All digits plus separators */
    if( len == 8 )
        sscanf( string, "%d%*c%d%*c%d", &hour, &min, &sec );

    /*  All digits, no separators */
    else if( len == 6 )
        sscanf( string, "%02d%02d%02d", &hour, &min, &sec );

    /*  Four digits, one separator */
    else if( len == 5 )
        sscanf( string, "%d%*c%d", &hour, &min );

    /*  Hour and minute, no separators */
    else if( len == 4 )
        sscanf( string, "%02d%02d", &hour, &min );

    /*  Hour only */
    else if( len == 2 )
        hour = atoi( string );

    /*  Anything else */
    else
    {
        printf("ERROR:  Incorrect time format\n");
        return( false );
    }


    /*  Set the time according to the given hour/min/sec */
    gTimes[ id ]->set_time_tr( hour, min, sec );

    return( true );
}



/*
--------------------------------------------------------------------------------
                                   PRINT_HELP
--------------------------------------------------------------------------------
*/
void print_help( void )
{
    printf("Usage:  eAlarm [Option/Time]\n\n");

    printf("Options:\n");
    printf("  -h or --help:\t\tPrint this help text\n");
    printf("  -t or --test:\t\tRun a test alarm\n");
    printf("  --version:\t\tPrint version and author info\n\n");
}



/*
--------------------------------------------------------------------------------
                                 PRINT_VERSION
--------------------------------------------------------------------------------
*/
void print_version( void )
{
    printf("eAlarm version %s\n", (VERSION) );
    printf("James Hendrie <hendrie.james@gmail.com>\n");
}



/*
--------------------------------------------------------------------------------
                                   TEST_ALARM
--------------------------------------------------------------------------------
*/
void test_alarm( void )
{
    /*  Save alarm time */
    gTimes[ TIME_ALARM ]->save_time();

    /*  Set the alarm to five seconds into the future */
    gTimes[ TIME_ALARM ]->set_time( now + 5 );

    /*  Switch to alarm screen */
    currentScreen = SCREEN_ALARM;

    /*  Init alarm screen */
    init_alarm_screen();

    testing = false;
}



/*
--------------------------------------------------------------------------------
                                  ARG_TO_TIME
--------------------------------------------------------------------------------
*/
bool arg_to_time( const char *string )
{
    /*  If we couldn't convert the argument to a time, bail */
    if( ! string_to_time( string, TIME_ALARM ) )
    {
        printf("ERROR:  Unable to convert string '%s' to time\n", string );
        return( false );
    }

    /*  Switch to alarm screen */
    currentScreen = SCREEN_ALARM;

    /*  Init alarm screen */
    init_alarm_screen();

    /*  Toggle argToTime */
    argToTime = false;


    return( true );
}


/*
================================================================================
--------------------------------------------------------------------------------

                                      MAIN

--------------------------------------------------------------------------------
================================================================================
*/
int main( int argc, char *argv[] )
{
    /*  Argument handling */
    if( argc > 1 )
    {
        /*  If they want help */
        if( ( strncmp( argv[1], "--help", 6 ) == 0 ) ||
            ( strncmp( argv[1], "-h", 2 ) == 0 ) )
        {
            print_help();
            return(0);
        }

        /*  If they want version and author info */
        else if( strncmp( argv[1], "--version", 9 ) == 0 )
        {
            print_version();
            return(0);
        }

        /*  If they want to run a test alarm */
        else if( ( strncmp( argv[1], "--test", 6) == 0 ) ||
                ( strncmp( argv[1], "-t", 2 ) == 0 ) )
        {
            testing = true;
        }

        /*  They put a number as the time */
        else
        {
            argToTime = true;
        }

    }


    /*  If we can't init */
    if( init() == false )
    {
        printf("ERROR:  Could not init\n");
        return(1);
    }

    /*  Init the times */
    init_times();

    /*  Find and read from the config file */
    if( ! init_config() )
    {
        printf("ERROR:  Could not init config\n");
        return( 1 );
    }

    /*  If we can't load the media */
    if( load_media() == false )
    {
        printf("ERROR:  Could not load media\n");
        return(1);
    }

    /*  Init preset buttons */
    init_time_buttons();

    /*  Init the buttons */
    init_buttons();

    /*  Init system time */
    update_system_time();

    /*  Configure layout */
    layout_main_screen();
    layout_time_screen();
    layout_help_screen();
    layout_about_screen();
    layout_alarm_screen();

    /*  Our event handler */
    SDL_Event e;


    /*  While the user hasn't decided to quit */
    while( quitting == false )
    {
        /*  System time and currently set alarm time */
        now = gTimes[ TIME_SYSTEM ]->get_time();
        set = gTimes[ TIME_ALARM  ]->get_time();

        /*  Check to see if we're testing the alarm */
        if( testing )
            test_alarm();

        /*  Check to see if we're doing the argToTime thing */
        if( argToTime )
        {
            /*  If arg_to_time fails, quit the program */
            if( ! arg_to_time( argv[ 1 ] ) )
                quitting = true;
        }

        /*  Handle events */
        while( SDL_PollEvent( &e ) != 0 )
        {
            /*  Reset the tooltip at the top of the loop */
            currentToolTip = -1;

            /*  If the user clicks the X button */
            if( e.type == SDL_QUIT )
                quitting = true;

            /*  Mouse events */
            else if( e.type == SDL_MOUSEBUTTONDOWN ||
                    e.type == SDL_MOUSEBUTTONUP || SDL_MOUSEMOTION )
            {
                /*  Pass the event to the screen-appropriate buttons */
                switch( currentScreen )
                {
                    case SCREEN_MAIN:   //  MAIN screen
                        handle_main_screen( e );
                        break;

                    case SCREEN_HELP:   //  HELP screen
                        handle_main_screen( e );
                        break;

                    case SCREEN_ABOUT:  //  ABOUT screen
                        handle_main_screen( e );
                        break;
                        
                    case SCREEN_TIME:   //  TIME screen
                        handle_time_screen( e );
                        break;

                }
            }


            /*  Keyboard events */
            if( e.type == SDL_KEYDOWN )
            {
                /*  'Q' will quit from anywhere in the program */
                if( e.key.keysym.sym == SDLK_q )
                    quitting = true;

                switch( currentScreen )
                {
                    case SCREEN_MAIN:   //  MAIN screen
                        handle_main_screen( e );
                        break;

                    case SCREEN_HELP:   //  HELP screen
                        handle_main_screen( e );
                        break;

                    case SCREEN_ABOUT:  //  ABOUT screen
                        handle_main_screen( e );
                        break;

                    case SCREEN_TIME:       //  TIME screen
                        handle_time_screen( e );
                        break;

                    case SCREEN_ALARM:      //  ALARM screen
                        handle_alarm_screen( e );
                        break;
                }
            }
        }

        /*  Set the render clear color */
        SDL_SetRenderDrawColor( gRenderer, 0, 0, 0, 0xff );

        /*  Clear the screen */
        SDL_RenderClear( gRenderer );

        /*  Set this for the box preset outlines */
        SDL_SetRenderDrawColor( gRenderer, 0xff, 0xff, 0xff, 0xff );


        /*  Render stuff to the screen */
        switch( currentScreen )
        {
            case SCREEN_MAIN:       //  MAIN screen

                /*  Render the info text for the main screen */
                render_text_main_screen();

                /*  Render main elements */
                render_main_screen();
                break;

            case SCREEN_ABOUT:      //  ABOUT screen

                render_text_about_screen();
                render_main_screen();

                break;

            case SCREEN_HELP:       //  HELP screen
                render_text_help_screen();
                render_main_screen();
                break;

            case SCREEN_TIME:       //  TIME screen
                render_time_screen();
                break;

            case SCREEN_ALARM:      //  ALARM screen
                check_alarm();
                render_alarm_screen();
                break;

        }


        /*  Update the screen */
        SDL_RenderPresent( gRenderer );

        /*  Basic FPS delay */
        SDL_Delay( 1000 / FPS );
    }

    /*  Close out SDL */
    close();

    return(0);
}
